@@ -4,8 +4,6 @@
 
 #include "performancedb.hxx"
 
-#include <boost/foreach.hpp>
-
 #include <simgear/sg_inlines.h>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
@@ -13,13 +11,12 @@
 #include <simgear/structure/exception.hxx>
 
 #include <Main/globals.hxx>
+#include <cstring>
 #include <iostream>
-#include <fstream>
 
 #include "performancedata.hxx"
 
 using std::string;
-using std::cerr;
 
 PerformanceDB::PerformanceDB()
 {
@@ -73,7 +70,7 @@ PerformanceData* PerformanceDB::getDataFor(const string& acType, const string& a
     if (it != _db.end()) {
         return it->second;
     }
-    
+
     const string& alias = findAlias(acType);
     it = _db.find(alias);
     if (it != _db.end()) {
@@ -111,10 +108,10 @@ void PerformanceDB::load(const SGPath& filename)
 {
     SGPropertyNode root;
     try {
-        readProperties(filename.str(), &root);
+        readProperties(filename, &root);
     } catch (const sg_exception &) {
         SG_LOG(SG_AI, SG_ALERT,
-            "Error reading AI aircraft performance database: " << filename.str());
+            "Error reading AI aircraft performance database: " << filename);
         return;
     }
 
@@ -131,13 +128,13 @@ void PerformanceDB::load(const SGPath& filename)
                        "Error reading AI aircraft performance database: unknown base type " << baseName);
                 return;
               }
-              
+
               // clone base data to 'inherit' from it
-              data = new PerformanceData(baseData); 
+              data = new PerformanceData(baseData);
             } else {
               data = new PerformanceData;
             }
-          
+
             data->initFromProps(db_node);
             const string& name  = db_node->getStringValue("type", "heavy_jet");
             registerPerformanceData(name, data);
@@ -147,8 +144,8 @@ void PerformanceDB::load(const SGPath& filename)
                 SG_LOG(SG_AI, SG_ALERT, "performance DB alias entry with no <alias> definition");
                 continue;
             }
-          
-            BOOST_FOREACH(SGPropertyNode* matchNode, db_node->getChildren("match")) {
+
+            for (auto matchNode : db_node->getChildren("match")) {
                 const string& match(matchNode->getStringValue());
                 _aliases.push_back(StringPair(match, alias));
             }
@@ -160,14 +157,17 @@ void PerformanceDB::load(const SGPath& filename)
 
 const string& PerformanceDB::findAlias(const string& acType) const
 {
-    BOOST_FOREACH(const StringPair& alias, _aliases) {
+    for (const auto& alias : _aliases) {
         if (acType.find(alias.first) == 0) { // matched!
             return alias.second;
         }
     } // of alias iteration
-  
+
     static const string empty;
     return empty;
 }
 
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<PerformanceDB> registrantPerformanceDB(
+    SGSubsystemMgr::POST_FDM);
