@@ -36,6 +36,7 @@
 #include "runwayprefs.hxx"
 
 #include <Navaids/NavDataCache.hxx>
+#include <Main/sentryIntegration.hxx>
 
 using std::string;
 
@@ -49,17 +50,51 @@ void XMLLoader::load(FGGroundNetwork* net)
     return;
   }
 
-  SG_LOG(SG_NAVAID, SG_INFO, "reading groundnet data from " << path);
+  SG_LOG(SG_NAVAID, SG_DEBUG, "reading groundnet data from " << path);
   SGTimeStamp t;
   t.stamp();
   try {
       FGGroundNetXMLLoader visitor(net);
       readXML(path, visitor);
+
+      if (visitor.hasErrors() && fgGetBool("/sim/terrasync/enabled")) {
+          flightgear::updateSentryTag("ground-net", net->airport()->ident());
+          flightgear::sentryReportException("Ground-net load error", path.utf8Str());
+      }
   } catch (sg_exception& e) {
-    SG_LOG(SG_NAVAID, SG_INFO, "parsing groundnet XML failed:" << e.getFormattedMessage());
+    SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
   }
 
-  SG_LOG(SG_NAVAID, SG_INFO, "parsing groundnet XML took " << t.elapsedMSec());
+  SG_LOG(SG_NAVAID, SG_DEBUG, "parsing groundnet XML took " << t.elapsedMSec());
+}
+
+void XMLLoader::loadFromStream(FGGroundNetwork* net, std::istream& inData)
+{
+  try {
+      FGGroundNetXMLLoader visitor(net);
+      readXML(inData, visitor);
+
+      if (visitor.hasErrors()) {
+          flightgear::sentryReportException("Ground-net load error", {});
+      }
+  } catch (sg_exception& e) {
+    SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
+  }
+}
+
+void XMLLoader::loadFromPath(FGGroundNetwork* net, const SGPath& path)
+{
+  try {
+      FGGroundNetXMLLoader visitor(net);
+      readXML(path, visitor);
+
+      if (visitor.hasErrors() && fgGetBool("/sim/terrasync/enabled")) {
+          flightgear::updateSentryTag("ground-net", net->airport()->ident());
+          flightgear::sentryReportException("Ground-net load error", path.utf8Str());
+      }
+  } catch (sg_exception& e) {
+    SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
+  }
 }
 
 void XMLLoader::load(FGRunwayPreference* p) {
@@ -105,8 +140,15 @@ bool XMLLoader::loadAirportXMLDataIntoVisitor(const string& aICAO,
     return false;
   }
 
-  SG_LOG(SG_GENERAL, SG_DEBUG, "loadAirportXMLDataIntoVisitor: loading from " << path);
-  readXML(path, aVisitor);
-  return true;
+    bool readXMLOk = true;
+    try {
+        SG_LOG(SG_NAVAID, SG_DEBUG, "loadAirportXMLDataIntoVisitor: loading from " << path);
+        readXML(path, aVisitor);
+    } catch (sg_exception& e) {
+        readXMLOk = false;
+        SG_LOG(SG_NAVAID, SG_WARN, "XML errors trying to read:" << path);
+    }
+    
+    return readXMLOk;
 }
 
