@@ -9,21 +9,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -42,9 +42,6 @@ COMMENTS, REFERENCES,  and NOTES
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <iterator>
-#include <cstdlib>
 #include <iomanip>
 
 #include "FGFDMExec.h"
@@ -53,17 +50,14 @@ INCLUDES
 #include "models/FGFCS.h"
 #include "models/FGPropulsion.h"
 #include "models/FGMassBalance.h"
-#include "models/FGGroundReactions.h"
 #include "models/FGExternalReactions.h"
 #include "models/FGBuoyantForces.h"
 #include "models/FGAerodynamics.h"
 #include "models/FGInertial.h"
 #include "models/FGAircraft.h"
 #include "models/FGAccelerations.h"
-#include "models/FGPropagate.h"
 #include "models/FGAuxiliary.h"
 #include "models/FGInput.h"
-#include "models/FGOutput.h"
 #include "initialization/FGTrim.h"
 #include "input_output/FGScript.h"
 #include "input_output/FGXMLFileRead.h"
@@ -72,9 +66,6 @@ using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGFDMExec.cpp,v 1.194 2017/03/03 23:00:39 bcoconni Exp $");
-IDENT(IdHdr,ID_FDMEXEC);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -82,13 +73,13 @@ CLASS IMPLEMENTATION
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 // Constructor
 
-FGFDMExec::FGFDMExec(FGPropertyManager* root, unsigned int* fdmctr) : Root(root), FDMctr(fdmctr)
+FGFDMExec::FGFDMExec(FGPropertyManager* root, unsigned int* fdmctr)
+  : Root(root), FDMctr(fdmctr)
 {
   Frame           = 0;
-  Error           = 0;
-  IC              = 0;
-  Trim            = 0;
-  Script          = 0;
+  IC              = nullptr;
+  Trim            = nullptr;
+  Script          = nullptr;
   disperse        = 0;
 
   RootDir = "";
@@ -169,6 +160,7 @@ FGFDMExec::FGFDMExec(FGPropertyManager* root, unsigned int* fdmctr) : Root(root)
   instance->Tie("simulation/disperse", this, &FGFDMExec::GetDisperse);
   instance->Tie("simulation/randomseed", this, (iPMF)&FGFDMExec::SRand, &FGFDMExec::SRand, false);
   instance->Tie("simulation/terminate", (int *)&Terminate);
+  instance->Tie("simulation/pause", (int *)&holding);
   instance->Tie("simulation/sim-time-sec", this, &FGFDMExec::GetSimTime);
   instance->Tie("simulation/dt", this, &FGFDMExec::GetDeltaT);
   instance->Tie("simulation/jsbsim-debug", this, &FGFDMExec::GetDebugLevel, &FGFDMExec::SetDebugLevel);
@@ -299,8 +291,6 @@ bool FGFDMExec::DeAllocate(void)
   delete IC;
   delete Trim;
 
-  Error       = 0;
-
   modelLoaded = false;
   return modelLoaded;
 }
@@ -353,8 +343,7 @@ void FGFDMExec::LoadInputs(unsigned int idx)
   case eInput:
     break;
   case eInertial:
-    Inertial->in.Radius        = Propagate->GetRadius();
-    Inertial->in.Latitude      = Propagate->GetLatitude();
+    Inertial->in.Position      = Propagate->GetLocation();
     break;
   case eAtmosphere:
     Atmosphere->in.altitudeASL = Propagate->GetAltitudeASL();
@@ -395,11 +384,8 @@ void FGFDMExec::LoadInputs(unsigned int idx)
     Auxiliary->in.SinTht       = Propagate->GetSinEuler(eTht);
     Auxiliary->in.CosPhi       = Propagate->GetCosEuler(ePhi);
     Auxiliary->in.SinPhi       = Propagate->GetSinEuler(ePhi);
-    Auxiliary->in.Psi          = Propagate->GetEuler(ePsi);
     Auxiliary->in.TotalWindNED = Winds->GetTotalWindNED();
     Auxiliary->in.TurbPQR      = Winds->GetTurbPQR();
-    Auxiliary->in.WindPsi      = Winds->GetWindPsi();
-    Auxiliary->in.Vwind        = Winds->GetTotalWindNED().Magnitude();
     break;
   case eSystems:
     // Dynamic inputs come into the components that FCS manages through properties
@@ -468,7 +454,7 @@ void FGFDMExec::LoadInputs(unsigned int idx)
     BuoyantForces->in.Density     = Atmosphere->GetDensity();
     BuoyantForces->in.Pressure    = Atmosphere->GetPressure();
     BuoyantForces->in.Temperature = Atmosphere->GetTemperature();
-    BuoyantForces->in.gravity     = Inertial->gravity();
+    BuoyantForces->in.gravity     = Inertial->GetGravity().Magnitude();
     break;
   case eMassBalance:
     MassBalance->in.GasInertia  = BuoyantForces->GetGasMassInertia();
@@ -501,8 +487,7 @@ void FGFDMExec::LoadInputs(unsigned int idx)
     Accelerations->in.GroundMoment  = GroundReactions->GetMoments();
     Accelerations->in.Force    = Aircraft->GetForces();
     Accelerations->in.GroundForce   = GroundReactions->GetForces();
-    Accelerations->in.GAccel   = Inertial->GetGAccel(Propagate->GetRadius());
-    Accelerations->in.J2Grav  = Inertial->GetGravityJ2(Propagate->GetLocation());
+    Accelerations->in.vGravAccel = Inertial->GetGravity();
     Accelerations->in.vPQRi    = Propagate->GetPQRi();
     Accelerations->in.vPQR     = Propagate->GetPQR();
     Accelerations->in.vUVW     = Propagate->GetUVW();
@@ -540,7 +525,6 @@ void FGFDMExec::LoadModelConstants(void)
   Aerodynamics->in.Wingspan      = Aircraft->GetWingSpan();
   Auxiliary->in.Wingspan         = Aircraft->GetWingSpan();
   Auxiliary->in.Wingchord        = Aircraft->Getcbar();
-  Auxiliary->in.PitotAngle       = Aircraft->GetPitotAngle();
   GroundReactions->in.vXYZcg     = MassBalance->GetXYZcg();
 
   LoadPlanetConstants();
@@ -684,8 +668,8 @@ bool FGFDMExec::LoadModel(const string& model, bool addModelToPath)
   modelName = model; // Set the class modelName attribute
 
   if( AircraftPath.isNull() || EnginePath.isNull() || SystemsPath.isNull()) {
-    cerr << "Error: attempted to load aircraft with undefined ";
-    cerr << "aircraft, engine, and system paths" << endl;
+    cerr << "Error: attempted to load aircraft with undefined "
+         << "aircraft, engine, and system paths" << endl;
     return false;
   }
 
@@ -956,10 +940,10 @@ void FGFDMExec::PrintPropertyCatalog(void)
 void FGFDMExec::PrintSimulationConfiguration(void) const
 {
   cout << endl << "Simulation Configuration" << endl << "------------------------" << endl;
-  cout << MassBalance->Name << endl;
-  cout << GroundReactions->Name << endl;
-  cout << Aerodynamics->Name << endl;
-  cout << Propulsion->Name << endl;
+  cout << MassBalance->GetName() << endl;
+  cout << GroundReactions->GetName() << endl;
+  cout << Aerodynamics->GetName() << endl;
+  cout << Propulsion->GetName() << endl;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1145,7 +1129,9 @@ void FGFDMExec::DoTrim(int mode)
 
   FGTrim trim(this, (JSBSim::TrimMode)mode);
   bool success = trim.DoTrim();
-  trim.Report();
+
+  if (debug_lvl > 0)
+    trim.Report();
 
   if (!success)
     throw("Trim Failed");
@@ -1212,11 +1198,7 @@ void FGFDMExec::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
 }
-
-
