@@ -7,21 +7,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 HISTORY
 --------------------------------------------------------------------------------
@@ -41,20 +41,9 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <vector>
-#include <string>
-
-#include "FGJSBBase.h"
-#include "input_output/FGPropertyManager.h"
 #include "models/FGPropagate.h"
-#include "math/FGColumnVector3.h"
 #include "models/FGOutput.h"
-
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_FDMEXEC "$Id: FGFDMExec.h,v 1.105 2016/04/16 12:24:39 bcoconni Exp $"
+#include "math/FGTemplateFunc.h"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -100,16 +89,16 @@ CLASS DOCUMENTATION
     result = fdmex->LoadModel( ... );
     @endcode
 
-    When an aircraft model is loaded, the config file is parsed and for each of the
-    sections of the config file (propulsion, flight control, etc.) the
+    When an aircraft model is loaded, the config file is parsed and for each of
+    the sections of the config file (propulsion, flight control, etc.) the
     corresponding Load() method is called (e.g. FGFCS::Load()).
 
     Subsequent to the creation of the executive and loading of the model,
     initialization is performed. Initialization involves copying control inputs
-    into the appropriate JSBSim data storage locations, configuring it for the set
-    of user supplied initial conditions, and then copying state variables from
-    JSBSim. The state variables are used to drive the instrument displays and to
-    place the vehicle model in world space for visual rendering:
+    into the appropriate JSBSim data storage locations, configuring it for the
+    set of user supplied initial conditions, and then copying state variables
+    from JSBSim. The state variables are used to drive the instrument displays
+    and to place the vehicle model in world space for visual rendering:
 
     @code
     copy_to_JSBsim(); // copy control inputs to JSBSim
@@ -126,13 +115,13 @@ CLASS DOCUMENTATION
     @endcode
 
     JSBSim can be used in a standalone mode by creating a compact stub program
-    that effectively performs the same progression of steps as outlined above for
-    the integrated version, but with two exceptions. First, the copy_to_JSBSim()
-    and copy_from_JSBSim() functions are not used because the control inputs are
-    handled directly by the scripting facilities and outputs are handled by the
-    output (data logging) class. Second, the name of a script file can be supplied
-    to the stub program. Scripting (see FGScript) provides a way to supply command
-    inputs to the simulation:
+    that effectively performs the same progression of steps as outlined above
+    for the integrated version, but with two exceptions. First, the
+    copy_to_JSBSim() and copy_from_JSBSim() functions are not used because the
+    control inputs are handled directly by the scripting facilities and outputs
+    are handled by the output (data logging) class. Second, the name of a script
+    file can be supplied to the stub program. Scripting (see FGScript) provides
+    a way to supply command inputs to the simulation:
 
     @code
     FDMExec = new JSBSim::FGFDMExec();
@@ -178,7 +167,7 @@ CLASS DOCUMENTATION
                                 property actually maps toa function call of DoTrim().
 
     @author Jon S. Berndt
-    @version $Revision: 1.105 $
+    @version $Revision: 1.106 $
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -284,8 +273,8 @@ public:
       @param addModelToPath set to true to add the model name to the
       AircraftPath, defaults to true
       @return true if successful */
-  bool LoadModel(const std::string& AircraftPath, const std::string& EnginePath,
-                 const std::string& SystemsPath, const std::string& model,
+  bool LoadModel(const SGPath& AircraftPath, const SGPath& EnginePath,
+                 const SGPath& SystemsPath, const std::string& model,
                  bool addModelToPath = true);
 
   /** Loads an aircraft model.  The paths to the aircraft and engine
@@ -310,24 +299,33 @@ public:
                       the file specified in the script will be used. If an initialization file 
                       is not given in either place, an error will result.
       @return true if successfully loads; false otherwise. */
-  bool LoadScript(const std::string& Script, double deltaT=0.0,
-                  const std::string& initfile="");
+  bool LoadScript(const SGPath& Script, double deltaT=0.0,
+                  const SGPath& initfile=SGPath());
 
   /** Sets the path to the engine config file directories.
       @param path path to the directory under which engine config
       files are kept, for instance "engine"  */
-  bool SetEnginePath(const std::string& path)   { EnginePath = RootDir + path; return true; }
+  bool SetEnginePath(const SGPath& path) {
+    EnginePath = GetFullPath(path);
+    return true;
+  }
 
   /** Sets the path to the aircraft config file directories.
       @param path path to the aircraft directory. For instance:
       "aircraft". Under aircraft, then, would be directories for various
       modeled aircraft such as C172/, x15/, etc.  */
-  bool SetAircraftPath(const std::string& path) { AircraftPath = RootDir + path; return true; }
+  bool SetAircraftPath(const SGPath& path) {
+    AircraftPath = GetFullPath(path);
+    return true;
+  }
   
   /** Sets the path to the systems config file directories.
       @param path path to the directory under which systems config
       files are kept, for instance "systems"  */
-  bool SetSystemsPath(const std::string& path)   { SystemsPath = RootDir + path; return true; }
+  bool SetSystemsPath(const SGPath& path) {
+    SystemsPath = GetFullPath(path);
+    return true;
+  }
   
   /// @name Top-level executive State and Model retrieval mechanism
   ///@{
@@ -378,13 +376,13 @@ public:
   ///@}
 
   /// Retrieves the engine path.
-  const std::string& GetEnginePath(void)    {return EnginePath;}
+  const SGPath& GetEnginePath(void)    {return EnginePath;}
   /// Retrieves the aircraft path.
-  const std::string& GetAircraftPath(void)  {return AircraftPath;}
+  const SGPath& GetAircraftPath(void)  {return AircraftPath;}
   /// Retrieves the systems path.
-  const std::string& GetSystemsPath(void)   {return SystemsPath;}
+  const SGPath& GetSystemsPath(void)   {return SystemsPath;}
   /// Retrieves the full aircraft path name.
-  const std::string& GetFullAircraftPath(void) {return FullAircraftPath;}
+  const SGPath& GetFullAircraftPath(void) {return FullAircraftPath;}
 
   /** Retrieves the value of a property.
       @param property the name of the property
@@ -428,8 +426,8 @@ public:
       be logged.
       @param fname the filename of an output directives file.
     */
-  bool SetOutputDirectives(const std::string& fname)
-  {return Output->SetDirectivesFile(RootDir + fname);}
+  bool SetOutputDirectives(const SGPath& fname)
+  { return Output->SetDirectivesFile(GetFullPath(fname)); }
 
   /** Forces the specified output object to print its items once */
   void ForceOutput(int idx=0) { Output->ForceOutput(idx); }
@@ -550,11 +548,11 @@ public:
 
   /** Sets the root directory where JSBSim starts looking for its system directories.
       @param rootDir the string containing the root directory. */
-  void SetRootDir(const std::string& rootDir) {RootDir = rootDir;}
+  void SetRootDir(const SGPath& rootDir) {RootDir = rootDir;}
 
   /** Retrieves the Root Directory.
       @return the string representing the root (base) JSBSim directory. */
-  const std::string& GetRootDir(void) const {return RootDir;}
+  const SGPath& GetRootDir(void) const {return RootDir;}
 
   /** Increments the simulation time if not in Holding mode. The Frame counter
       is also incremented.
@@ -589,8 +587,15 @@ public:
   */
   bool GetHoldDown(void) const {return HoldDown;}
 
+  FGTemplateFunc* GetTemplateFunc(const std::string& name) {
+    return TemplateFunctions.count(name) ? TemplateFunctions[name] : nullptr;
+  }
+
+  void AddTemplateFunc(const std::string& name, Element* el) {
+    TemplateFunctions[name] = new FGTemplateFunc(this, el);
+  }
+
 private:
-  int Error;
   unsigned int Frame;
   unsigned int IdFDM;
   int disperse;
@@ -606,13 +611,13 @@ private:
   bool modelLoaded;
   bool IsChild;
   std::string modelName;
-  std::string AircraftPath;
-  std::string FullAircraftPath;
-  std::string EnginePath;
-  std::string SystemsPath;
+  SGPath AircraftPath;
+  SGPath FullAircraftPath;
+  SGPath EnginePath;
+  SGPath SystemsPath;
   std::string CFGVersion;
   std::string Release;
-  std::string RootDir;
+  SGPath RootDir;
 
   // Standard Model pointers - shortcuts for internal executive use only.
   FGPropagate* Propagate;
@@ -646,12 +651,14 @@ private:
 
   bool HoldDown;
 
-  // The FDM counter is used to give each child FDM an unique ID. The root FDM has the ID 0
+  // The FDM counter is used to give each child FDM an unique ID. The root FDM
+  // has the ID 0
   unsigned int*      FDMctr;
 
   std::vector <std::string> PropertyCatalog;
   std::vector <childData*> ChildFDMList;
   std::vector <FGModel*> Models;
+  std::map<std::string, FGTemplateFunc_ptr> TemplateFunctions;
 
   bool ReadFileHeader(Element*);
   bool ReadChild(Element*);
@@ -664,6 +671,12 @@ private:
   bool Allocate(void);
   bool DeAllocate(void);
   int GetDisperse(void) const {return disperse;}
+  SGPath GetFullPath(const SGPath& name) {
+    if (name.isRelative())
+      return RootDir/name.utf8Str();
+    else
+      return name;
+  }
 
   void Debug(int from);
 };
