@@ -20,6 +20,8 @@
 
 #include "LocalAircraftCache.hxx"
 
+#include <cassert>
+
 #include <QDir>
 #include <QThread>
 #include <QMutex>
@@ -29,28 +31,39 @@
 #include <QSettings>
 #include <QDebug>
 
-#include <Main/globals.hxx>
 #include <Include/version.h>
+#include <Main/globals.hxx>
+#include <Main/locale.hxx>
+#include <Main/sentryIntegration.hxx>
 
+#include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/structure/exception.hxx>
 
-static quint32 CACHE_VERSION = 8;
+static quint32 CACHE_VERSION = 13;
 
-const int STANDARD_THUMBNAIL_HEIGHT = 128;
-//const int STANDARD_THUMBNAIL_WIDTH = 172;
+const std::vector<QByteArray> static_localizedStringTags = {"name", "desc"};
 
-AircraftItem::AircraftItem()
+QDataStream& operator<<(QDataStream& ds, const AircraftItem::LocalizedStrings& ls)
 {
+    ds << ls.locale << ls.strings;
+    return ds;
 }
 
-AircraftItem::AircraftItem(QDir dir, QString filePath)
+QDataStream& operator>>(QDataStream& ds, AircraftItem::LocalizedStrings& ls)
+{
+    ds >> ls.locale >> ls.strings;
+    return ds;
+}
+
+bool AircraftItem::initFromFile(QDir dir, QString filePath)
 {
     SGPropertyNode root;
     readProperties(filePath.toStdString(), &root);
 
     if (!root.hasChild("sim")) {
-        throw sg_io_exception(std::string("Malformed -set.xml file"), filePath.toStdString());
+        qWarning() << "-set.xml has no <sim> element" << filePath;
+        return false;
     }
 
     SGPropertyNode_ptr sim = root.getNode("sim");
@@ -59,24 +72,24 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
     pathModTime = QFileInfo(path).lastModified();
     if (sim->getBoolValue("exclude-from-gui", false)) {
         excluded = true;
-        return;
+        return false;
     }
 
-    description = sim->getStringValue("description");
-    authors =  sim->getStringValue("author");
+    LocalizedStrings ls;
+    ls.locale = "en";
+    ls.strings["name"] = QString::fromStdString(sim->getStringValue("description")).trimmed();
+    authors = sim->getStringValue("author");
 
     if (sim->hasChild("rating")) {
         SGPropertyNode_ptr ratingsNode = sim->getNode("rating");
-        ratings[0] = ratingsNode->getIntValue("FDM");
-        ratings[1] = ratingsNode->getIntValue("systems");
-        ratings[2] = ratingsNode->getIntValue("cockpit");
-        ratings[3] = ratingsNode->getIntValue("model");
-
+        for (int i=0; i< 4; ++i) {
+            ratings[i] = LocalAircraftCache::ratingFromProperties(ratingsNode, i);
+        }
     }
 
     if (sim->hasChild("long-description")) {
         // clean up any XML whitspace in the text.
-        longDescription = QString(sim->getStringValue("long-description")).simplified();
+        ls.strings["desc"] = QString(sim->getStringValue("long-description")).simplified();
     }
 
     if (sim->hasChild("variant-of")) {
@@ -100,8 +113,10 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
                 // could also consider vtol tag?
                 usesSeaports |= (strcmp(tagName, "seaplane") == 0);
                 usesSeaports |= (strcmp(tagName, "floats") == 0);
-
                 needsMaintenance |= (strcmp(tagName, "needs-maintenance") == 0);
+
+                // and actually store the tags
+                tags.push_back(QString::fromUtf8(tagName));
             }
         } // of tags iteration
     } // of set-xml has tags
@@ -125,6 +140,71 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
     if (sim->hasChild("minimum-fg-version")) {
         minFGVersion = sim->getStringValue("minimum-fg-version");
     }
+
+    homepageUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/home-page")));
+    supportUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/support")));
+    wikipediaUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/wikipedia")));
+
+    _localized.push_front(ls);
+    readLocalizedStrings(sim);
+    doLocalizeStrings();
+
+    return true;
+}
+
+void AircraftItem::readLocalizedStrings(SGPropertyNode_ptr simNode)
+{
+    if (!simNode->hasChild("localized"))
+        return;
+
+    auto localeNode = simNode->getChild("localized");
+    const auto num = localeNode->nChildren();
+    for (int i = 0; i < num; i++) {
+        const SGPropertyNode* c = localeNode->getChild(i);
+
+        LocalizedStrings ls;
+        ls.locale = QString::fromStdString(c->getNameString());
+        if (c->hasChild("description")) {
+            ls.strings["name"] = QString::fromStdString(c->getStringValue("description"));
+        }
+        if (c->hasChild("long-description")) {
+            ls.strings["desc"] = QString::fromStdString(c->getStringValue("long-description")).simplified();
+        }
+
+        _localized.push_back(ls);
+    }
+}
+
+void AircraftItem::doLocalizeStrings()
+{
+    // default strings are always at the front
+    _currentStrings = _localized.front().strings;
+
+    const auto lang = QString::fromStdString(globals->get_locale()->getPreferredLanguage());
+    // find the matching locale
+    auto it = std::find_if(_localized.begin(), _localized.end(), [lang](const LocalizedStrings& ls) {
+        return ls.locale == lang;
+    });
+
+    if (it == _localized.end())
+        return; // nothing else to do
+
+    for (auto t : static_localizedStringTags) {
+        if (it->strings.contains(t)) {
+            // copy the value we found
+            _currentStrings[t] = it->strings.value(t);
+        }
+    } // of strings iteration
+}
+
+QString AircraftItem::name() const
+{
+    return _currentStrings.value("name");
+}
+
+QString AircraftItem::description() const
+{
+    return _currentStrings.value("desc");
 }
 
 QString AircraftItem::baseName() const
@@ -141,12 +221,17 @@ void AircraftItem::fromDataStream(QDataStream& ds)
         return;
     }
 
-    ds >> description >> longDescription >> authors >> variantOf >> isPrimary;
+    ds >> authors >> variantOf >> isPrimary;
     for (int i=0; i<4; ++i) ds >> ratings[i];
     ds >> previews;
     ds >> thumbnailPath;
     ds >> minFGVersion;
     ds >> needsMaintenance >> usesHeliports >> usesSeaports;
+    ds >> homepageUrl >> supportUrl >> wikipediaUrl;
+    ds >> tags;
+    ds >> _localized;
+
+    doLocalizeStrings();
 }
 
 void AircraftItem::toDataStream(QDataStream& ds) const
@@ -156,29 +241,15 @@ void AircraftItem::toDataStream(QDataStream& ds) const
         return;
     }
 
-    ds << description << longDescription << authors << variantOf << isPrimary;
+    ds << authors << variantOf << isPrimary;
     for (int i=0; i<4; ++i) ds << ratings[i];
     ds << previews;
     ds << thumbnailPath;
     ds << minFGVersion;
     ds << needsMaintenance << usesHeliports << usesSeaports;
-}
-
-QPixmap AircraftItem::thumbnail(bool loadIfRequired) const
-{
-    if (m_thumbnail.isNull() && loadIfRequired) {
-        QFileInfo info(path);
-        QDir dir = info.dir();
-        if (dir.exists(thumbnailPath)) {
-            m_thumbnail.load(dir.filePath(thumbnailPath));
-            // resize to the standard size
-            if (m_thumbnail.height() > STANDARD_THUMBNAIL_HEIGHT) {
-                m_thumbnail = m_thumbnail.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT, Qt::SmoothTransformation);
-            }
-        }
-    }
-
-    return m_thumbnail;
+    ds << homepageUrl << supportUrl << wikipediaUrl;
+    ds << tags;
+    ds << _localized;
 }
 
 int AircraftItem::indexOfVariant(QUrl uri) const
@@ -195,6 +266,7 @@ int AircraftItem::indexOfVariant(QUrl uri) const
 
 QVariant AircraftItem::status(int variant)
 {
+    Q_UNUSED(variant)
     if (needsMaintenance) {
         return LocalAircraftCache::AircraftUnmaintained;
     }
@@ -210,6 +282,84 @@ QVariant AircraftItem::status(int variant)
 
 }
 
+namespace {
+
+static std::unique_ptr<LocalAircraftCache> static_cacheInstance;
+
+
+// ensure references to Aircraft/foo and <my-aircraft-dir>/foo are resolved. This happens when
+// aircraft reference a path (probably to themselves) in their -set.xml
+class ScanDirProvider : public simgear::ResourceProvider
+{
+public:
+    ScanDirProvider() : simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {}
+
+    ~ScanDirProvider() = default;
+
+    SGPath resolve(const std::string& aResource, SGPath& aContext) const override
+    {
+        Q_UNUSED(aContext)
+
+        SGPath ap = _currentAircraftPath / aResource;
+        if (ap.exists())
+            return ap;
+
+        string_list pieces(sgPathBranchSplit(aResource));
+        if ((pieces.size() < 3) || (pieces.front() != "Aircraft")) {
+            return SGPath{}; // not an Aircraft path
+        }
+
+        const std::string res(aResource, 9); // resource path with 'Aircraft/' removed
+        SGPath p = _currentScanPath / res;
+        if (p.exists())
+            return p;
+
+        return SGPath{};
+    }
+
+    void setCurrentPath(const SGPath& p)
+    {
+        _currentScanPath = p;
+    }
+
+    void setCurrentAircraftPath(const SGPath& p)
+    {
+        _currentAircraftPath = p;
+    }
+
+private:
+    SGPath _currentScanPath;
+    SGPath _currentAircraftPath;
+};
+
+class OtherAircraftDirsProvider : public simgear::ResourceProvider
+{
+public:
+    OtherAircraftDirsProvider() : simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {}
+
+    ~OtherAircraftDirsProvider() = default;
+
+    SGPath resolve(const std::string& aResource, SGPath& aContext) const override
+    {
+        if (aResource.find("Aircraft/") != 0) {
+            return SGPath{}; // not an aircraft path
+        }
+
+        const std::string res(aResource, 9); // resource path with 'Aircraft/' removed
+
+        Q_UNUSED(aContext)
+        QStringList paths = LocalAircraftCache::instance()->paths();
+        Q_FOREACH(auto p, paths) {
+            const auto sp = SGPath::fromUtf8(p.toUtf8().toStdString()) / res;
+           // qWarning() << "OADP: trying:" << QString::fromStdString(sp.utf8Str());
+            if (sp.exists())
+                return sp;
+        }
+
+        return SGPath{};
+    }
+};
+
 class AircraftScanThread : public QThread
 {
     Q_OBJECT
@@ -218,10 +368,14 @@ public:
         m_dirs(dirsToScan),
         m_done(false)
     {
+        auto rm = simgear::ResourceManager::instance();
+        m_currentScanDir.reset(new ScanDirProvider);
+        rm->addProvider(m_currentScanDir.get());
     }
 
     ~AircraftScanThread()
     {
+        simgear::ResourceManager::instance()->removeProvider(m_currentScanDir.get());
     }
 
     /** thread-safe access to items already scanned */
@@ -239,15 +393,19 @@ public:
     {
         m_done = true;
     }
+
 Q_SIGNALS:
     void addedItems();
 
 protected:
-    virtual void run()
+    void run() override
     {
+        flightgear::addSentryBreadcrumb("AircraftScan started", "info");
         readCache();
 
         Q_FOREACH(QString d, m_dirs) {
+            const auto p = SGPath::fromUtf8(d.toUtf8().toStdString());
+            m_currentScanDir->setCurrentPath(p);
             scanAircraftDir(QDir(d));
             if (m_done) {
                 return;
@@ -255,6 +413,7 @@ protected:
         }
 
         writeCache();
+        flightgear::addSentryBreadcrumb("AircraftScan finished", "info");
     }
 
 private:
@@ -291,7 +450,7 @@ private:
         QByteArray cacheData;
         {
             QDataStream ds(&cacheData, QIODevice::WriteOnly);
-            quint32 count = m_nextCache.count();
+            quint32 count = static_cast<quint32>(m_nextCache.count());
             ds << CACHE_VERSION << count;
 
             Q_FOREACH(AircraftItemPtr item, m_nextCache.values()) {
@@ -314,6 +473,11 @@ private:
             QMap<QString, AircraftItemPtr> baseAircraft;
             QList<AircraftItemPtr> variants;
 
+            // ensure aircraft dir is available to simgear::ResourceProvider
+            // otherwise some aircraft -set.xml includes fail
+            const auto p = SGPath::fromUtf8(child.absoluteFilePath().toUtf8().toStdString());
+            m_currentScanDir->setCurrentAircraftPath(p);
+
             Q_FOREACH(QFileInfo xmlChild, childDir.entryInfoList(filters, QDir::Files)) {
                 try {
                     QString absolutePath = xmlChild.absoluteFilePath();
@@ -325,7 +489,11 @@ private:
                         item = m_cachedItems.value(absolutePath);
                     } else {
                         // scan the cached item
-                        item = AircraftItemPtr(new AircraftItem(childDir, absolutePath));
+                        item = AircraftItemPtr(new AircraftItem);
+                        bool ok = item->initFromFile(childDir, absolutePath);
+                        if (!ok) {
+                            continue;
+                        }
                     }
 
                     m_nextCache[absolutePath] = item;
@@ -340,6 +508,8 @@ private:
                         variants.append(item);
                     }
                 } catch (sg_exception& e) {
+                    qWarning() << "Problems occurred while parsing" << xmlChild.absoluteFilePath() << "(skipping)"
+                               << "\n\t" << QString::fromStdString(e.what());
                     continue;
                 }
 
@@ -382,9 +552,20 @@ private:
     QMap<QString, AircraftItemPtr > m_nextCache;
 
     bool m_done;
+    std::unique_ptr<ScanDirProvider> m_currentScanDir;
 };
 
-std::unique_ptr<LocalAircraftCache> static_cacheInstance;
+} // of anonymous namespace
+
+
+class LocalAircraftCache::AircraftCachePrivate
+{
+public:
+    QStringList m_paths;
+    std::unique_ptr<AircraftScanThread> m_scanThread;
+    QVector<AircraftItemPtr> m_items;
+    std::unique_ptr<OtherAircraftDirsProvider> m_otherDirsProvider;
+};
 
 LocalAircraftCache* LocalAircraftCache::instance()
 {
@@ -395,76 +576,95 @@ LocalAircraftCache* LocalAircraftCache::instance()
     return static_cacheInstance.get();
 }
 
-LocalAircraftCache::LocalAircraftCache()
+void LocalAircraftCache::reset()
 {
+    static_cacheInstance.reset();
+}
 
+LocalAircraftCache::LocalAircraftCache() :
+    d(new AircraftCachePrivate)
+{
+    d->m_otherDirsProvider.reset(new OtherAircraftDirsProvider);
+    auto rm = simgear::ResourceManager::instance();
+    rm->addProvider(d->m_otherDirsProvider.get());
 }
 
 LocalAircraftCache::~LocalAircraftCache()
 {
     abandonCurrentScan();
+    if (simgear::ResourceManager::haveInstance()) {
+        simgear::ResourceManager::instance()->removeProvider(d->m_otherDirsProvider.get());
+    } else {
+        // resource manager will already have destroyed the provider. Awkward
+        // ownership model :(
+        d->m_otherDirsProvider.release();
+    }
 }
 
 void LocalAircraftCache::setPaths(QStringList paths)
 {
-    if (paths == m_paths) {
+    if (paths == d->m_paths) {
         return;
     }
 
-    m_items.clear();
+    d->m_items.clear();
     emit cleared();
-    m_paths = paths;
+    d->m_paths = paths;
+}
+
+QStringList LocalAircraftCache::paths() const
+{
+    return d->m_paths;
 }
 
 void LocalAircraftCache::scanDirs()
 {
     abandonCurrentScan();
-    m_items.clear();
+    d->m_items.clear();
 
-    QStringList dirs = m_paths;
+    QStringList dirs = d->m_paths;
 
-    Q_FOREACH(SGPath ap, globals->get_aircraft_paths()) {
+    for (SGPath ap : globals->get_aircraft_paths()) {
         dirs << QString::fromStdString(ap.utf8Str());
     }
 
-    SGPath rootAircraft(globals->get_fg_root());
-    rootAircraft.append("Aircraft");
+    SGPath rootAircraft = globals->get_fg_root() / "Aircraft";
     dirs << QString::fromStdString(rootAircraft.utf8Str());
 
-    m_scanThread.reset(new AircraftScanThread(dirs));
-    connect(m_scanThread.get(), &AircraftScanThread::finished, this,
+    d->m_scanThread.reset(new AircraftScanThread(dirs));
+    connect(d->m_scanThread.get(), &AircraftScanThread::finished, this,
             &LocalAircraftCache::onScanFinished);
     // force a queued connection here since we the scan thread object still
     // belongs to the same thread as us, and hence this would otherwise be
     // a direct connection
-    connect(m_scanThread.get(), &AircraftScanThread::addedItems,
+    connect(d->m_scanThread.get(), &AircraftScanThread::addedItems,
             this, &LocalAircraftCache::onScanResults,
             Qt::QueuedConnection);
-    m_scanThread->start();
+    d->m_scanThread->start();
 
     emit scanStarted();
 }
 
 int LocalAircraftCache::itemCount() const
 {
-    return m_items.size();
+    return d->m_items.size();
 }
 
 QVector<AircraftItemPtr> LocalAircraftCache::allItems() const
 {
-    return m_items;
+    return d->m_items;
 }
 
 AircraftItemPtr LocalAircraftCache::itemAt(int index) const
 {
-    return m_items.at(index);
+    return d->m_items.at(index);
 }
 
 int LocalAircraftCache::findIndexWithUri(QUrl aircraftUri) const
 {
     QString path = aircraftUri.toLocalFile();
-    for (int row=0; row < m_items.size(); ++row) {
-        const AircraftItemPtr item(m_items.at(row));
+    for (int row=0; row < d->m_items.size(); ++row) {
+        const AircraftItemPtr item(d->m_items.at(row));
         if (item->path == path) {
             return row;
         }
@@ -485,8 +685,8 @@ AircraftItemPtr LocalAircraftCache::primaryItemFor(AircraftItemPtr item) const
     if (!item || item->variantOf.isEmpty())
         return item;
 
-    for (int row=0; row < m_items.size(); ++row) {
-        const AircraftItemPtr p(m_items.at(row));
+    for (int row=0; row < d->m_items.size(); ++row) {
+        const AircraftItemPtr p(d->m_items.at(row));
         if (p->baseName() == item->variantOf) {
             return p;
         }
@@ -499,7 +699,7 @@ AircraftItemPtr LocalAircraftCache::findItemWithUri(QUrl aircraftUri) const
 {
     int index = findIndexWithUri(aircraftUri);
     if (index >= 0) {
-        return m_items.at(index);
+        return d->m_items.at(index);
     }
 
     return {};
@@ -507,32 +707,37 @@ AircraftItemPtr LocalAircraftCache::findItemWithUri(QUrl aircraftUri) const
 
 void LocalAircraftCache::abandonCurrentScan()
 {
-    if (m_scanThread) {
-        m_scanThread->setDone();
-        m_scanThread->wait(1000);
-        m_scanThread.reset();
-        qWarning() << Q_FUNC_INFO << "current scan abandonded";
+    if (d->m_scanThread) {
+        // don't fire onScanFinished when the thread ends
+        disconnect(d->m_scanThread.get(), &AircraftScanThread::finished, this,
+                &LocalAircraftCache::onScanFinished);
+
+        d->m_scanThread->setDone();
+        if (!d->m_scanThread->wait(2000)) {
+            qWarning() << Q_FUNC_INFO << "scan thread failed to terminate";
+        }
+        d->m_scanThread.reset();
     }
 }
 
 
 void LocalAircraftCache::onScanResults()
 {
-    if (!m_scanThread) {
+    if (!d->m_scanThread) {
         return;
     }
 
-    QVector<AircraftItemPtr> newItems = m_scanThread->items();
+    QVector<AircraftItemPtr> newItems = d->m_scanThread->items();
     if (newItems.isEmpty())
         return;
 
-    m_items+=newItems;
+    d->m_items+=newItems;
     emit addedItems(newItems.size());
 }
 
 void LocalAircraftCache::onScanFinished()
 {
-    m_scanThread.reset();
+    d->m_scanThread.reset();
     emit scanCompleted();
 }
 
@@ -562,4 +767,54 @@ bool LocalAircraftCache::isCandidateAircraftPath(QString path)
     return (setXmlCount > 0);
 }
 
+int LocalAircraftCache::ratingFromProperties(SGPropertyNode* node, int ratingIndex)
+{
+    const char* names[] = {"FDM", "systems", "cockpit", "model"};
+    assert((ratingIndex >= 0) && (ratingIndex < 4));
+    return node->getIntValue(names[ratingIndex]);
+}
+
+LocalAircraftCache::ParseSetXMLResult
+LocalAircraftCache::readAircraftProperties(const SGPath &setPath, SGPropertyNode_ptr props)
+{
+    // it woudld be race-y to touch the reosurce provider while the scan thread is running
+    // and our provider would confuse current-aircraft-dir lookups as well. Since we
+    // can't do thread-specific reosurce providers, we just bail here.
+    if (d->m_scanThread) {
+        return ParseSetXMLResult::Retry;
+    }
+
+    auto rm = simgear::ResourceManager::instance();
+
+    // ensure aircraft relative paths in the -set.xml parsing work
+
+    std::unique_ptr<ScanDirProvider> dp{new ScanDirProvider};
+    rm->addProvider(dp.get());
+
+    // we want to know the aircraft directory the aircraft lives in. This might
+    // be a manually added path (for local aircraft) or the install dir for the
+    // hangar (for packaged aircraft). Becuase -set.xml files are always found at
+    // /some/path/foobarAircraft/<aircraft-name>/some-set.xml, and the path we
+    // we want here is /some/path/foodbarAircraft, we use dirPath twice, and this
+    // works any kind of installed aircraft
+    const SGPath aircraftDirPath = setPath.dirPath().dirPath();
+    dp->setCurrentPath(aircraftDirPath);
+
+    dp->setCurrentAircraftPath(setPath);
+
+    ParseSetXMLResult result = ParseSetXMLResult::Failed;
+
+    try {
+        readProperties(setPath, props);
+        result = ParseSetXMLResult::Ok;
+    } catch (sg_exception& e) {
+        // malformed include or XML
+        qWarning() << "Problems occurred while parsing" << QString::fromStdString(setPath.utf8Str())
+                   << "\n\t" << QString::fromStdString(e.getFormattedMessage());
+    }
+
+    rm->removeProvider(dp.get());
+    return result;
+}
+
 #include "LocalAircraftCache.moc"
