@@ -20,10 +20,7 @@
 //
 // $Id$
 
-
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
@@ -56,17 +53,22 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/scene/tgdb/GroundLightManager.hxx>
 
-#include <osg/Texture>
 #include <osg/BufferObject>
+#include <osg/Texture>
+#include <osg/Version>
+#include <osgText/Font>
 
 #include <Viewer/fgviewer.hxx>
 #include "main.hxx"
 #include <Include/version.h>
+#include <Include/build.h>
 #include <Main/globals.hxx>
 #include <Main/fg_init.hxx>
 #include <Main/options.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/sentryIntegration.hxx>
 #include <GUI/MessageBox.hxx>
+#include <Navaids/NavDataCache.hxx>
 
 #include "fg_os.hxx"
 
@@ -74,13 +76,6 @@
 #include <GUI/QtLauncher.hxx>
 #endif
 
-#if defined(HAVE_CRASHRPT)
-	#include <CrashRpt.h>
-
-bool global_crashRptEnabled = false;
-
-#endif
-
 using std::cerr;
 using std::endl;
 
@@ -136,9 +131,22 @@ static void initFPE(bool)
 #if defined(SG_WINDOWS)
 int main ( int argc, char **argv );
 int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
-                             LPSTR lpCmdLine, int nCmdShow) {
+                             LPSTR lpCmdLine, int nCmdShow) 
+{
+	// convert wchar_t args to UTF-8 which is what we expect cross-platform
+    int     numArgs  = 0;
+	LPWSTR* wideArgs = CommandLineToArgvW(GetCommandLineW(), &numArgs);
 
-  main( __argc, __argv );
+	std::vector<char*> utf8Args;
+    utf8Args.resize(numArgs);
+
+	for (int a = 0; a < numArgs; ++a) {
+		const auto s = simgear::strutils::convertWStringToUtf8(wideArgs[a]);
+		// note we leak these (strudp calls malloc) but not a big concern
+        utf8Args[a] = strdup(s.c_str());
+	}
+
+	main(numArgs, utf8Args.data());
 }
 #endif
 
@@ -151,6 +159,7 @@ void segfault_handler(int signo) {
 
   fprintf(stderr, "Error: caught signal %d:\n", signo);
 
+  #ifndef __OpenBSD__
   size = backtrace(array, 128);
   if (size) {
     char** list = backtrace_symbols(array, size);
@@ -187,6 +196,7 @@ void segfault_handler(int signo) {
     free(fname);
     free(list);
   }
+  #endif
 
   std::abort();
 }
@@ -225,6 +235,12 @@ int _bootstrap_OSInit;
 // Main entry point; catch any exceptions that have made it this far.
 int main ( int argc, char **argv )
 {
+    // we don't want to accidently show a GUI box and block startup in
+    // non_GUI setups, so check this value early here, before options are
+    // processed
+    const bool headless = flightgear::Options::checkForArg(argc, argv, "disable-gui");
+    flightgear::setHeadlessMode(headless);
+
 #ifdef ENABLE_SIMD
   if (!detectSIMD()) {
     flightgear::fatalMessageBoxThenExit(
@@ -246,62 +262,21 @@ int main ( int argc, char **argv )
   char _hostname[256];
   gethostname(_hostname, 256);
   hostname = _hostname;
-
   signal(SIGPIPE, SIG_IGN);
-# ifndef NDEBUG
-  signal(SIGSEGV, segfault_handler);
-# endif
 #endif
 
   _bootstrap_OSInit = 0;
+    
+#if defined(HAVE_SENTRY)
+  flightgear::initSentry();
+#endif
 
-#if defined(HAVE_CRASHRPT)
-	// Define CrashRpt configuration parameters
-	CR_INSTALL_INFO info;
-	memset(&info, 0, sizeof(CR_INSTALL_INFO));
-	info.cb = sizeof(CR_INSTALL_INFO);
-	info.pszAppName = "FlightGear";
-	info.pszAppVersion = FLIGHTGEAR_VERSION;
-	info.pszEmailSubject = "FlightGear " FLIGHTGEAR_VERSION " crash report";
-	info.pszEmailTo = "fgcrash@goneabitbursar.com";
-	info.pszUrl = "http://fgfs.goneabitbursar.com/crashreporter/crashrpt.php";
-	info.uPriorities[CR_HTTP] = 3;
-	info.uPriorities[CR_SMTP] = 2;
-	info.uPriorities[CR_SMAPI] = 1;
-
-	// Install all available exception handlers
-	info.dwFlags |= CR_INST_ALL_POSSIBLE_HANDLERS;
-
-	// Restart the app on crash
-	info.dwFlags |= CR_INST_SEND_QUEUED_REPORTS;
-
-	// automatically install handlers for all threads
-	info.dwFlags |= CR_INST_AUTO_THREAD_HANDLERS;
-
-	// Define the Privacy Policy URL
-	info.pszPrivacyPolicyURL = "http://flightgear.org/crash-privacypolicy.html";
-
-	// Install crash reporting
-	int nResult = crInstall(&info);
-	if(nResult!=0) {
-		// don't warn about missing CrashRpt in developer builds
-		if (strcmp(FG_BUILD_TYPE, "Dev") != 0) {
-			char buf[1024];
-			crGetLastErrorMsg(buf, 1024);
-			flightgear::modalMessageBox("CrashRpt setup failed",
-				"Failed to setup crash-reporting engine, check the installation is not damaged.",
-				buf);
-		}
-	} else {
-		global_crashRptEnabled = true;
-
-		crAddProperty("hudson-build-id", HUDSON_BUILD_ID);
-		char buf[16];
-		::snprintf(buf, 16, "%d", HUDSON_BUILD_NUMBER);
-		crAddProperty("hudson-build-number", buf);
-    crAddProperty("git-revision", REVISION);
-    crAddProperty("build-type", FG_BUILD_TYPE);
-	}
+// if we're not using the normal crash-reported, install our
+// custom segfault handler on Linux, in debug builds
+#if !defined(SG_WINDOWS) && !defined(NDEBUG)
+    if (!flightgear::isSentryEnabled()) {
+        signal(SIGSEGV, segfault_handler);
+    }
 #endif
 
     initFPE(flightgear::Options::checkForArg(argc, argv, "enable-fpe"));
@@ -326,12 +301,17 @@ int main ( int argc, char **argv )
         sglog();
 
 
-#if (OPENSCENEGRAPH_MAJOR_VERSION == 3) && (OPENSCENEGRAPH_MINOR_VERSION < 5)
+#if OSG_VERSION_LESS_THAN(3, 5, 0)
         // similar to above, ensure some static maps inside OSG exist before
         // we register our at-exit handler, otherwise the statics are gone
         // when fg_terminate runs, which causes crashes.
         osg::Texture::getTextureObjectManager(0);
         osg::GLBufferObjectManager::getGLBufferObjectManager(0);
+
+        // ensure this is called early (and hence deleted) late,
+        // otherwise fgExitCleanup crahses: see
+        // Sentry FLIGHTEAR-M68
+        osgText::Font::getDefaultFont();
 #endif
         std::set_terminate(fg_terminate);
         atexit(fgExitCleanup);
@@ -347,14 +327,19 @@ int main ( int argc, char **argv )
             info = std::string("received from ") + t.getOrigin();
         flightgear::fatalMessageBoxWithoutExit(
           "Fatal exception", t.getFormattedMessage(), info);
+    } catch (const std::bad_alloc&) {
+        flightgear::fatalMessageBoxWithoutExit("Out of memory",
+        "FlightGear ran out of memory and must exit.", "Consider adjusting your settings or closing other applications.");
     } catch (const std::exception &e ) {
         flightgear::fatalMessageBoxWithoutExit("Fatal exception", e.what());
     } catch (const std::string &s) {
         flightgear::fatalMessageBoxWithoutExit("Fatal exception", s);
+    } catch (const flightgear::FatalErrorException&) {
+        // we already showed the message box, just carry on to exit
     } catch (const char *s) {
         std::cerr << "Fatal error (const char*): " << s << std::endl;
-
     } catch (...) {
+        flightgear::sentryReportException("Unknown main loop exception");
         std::cerr << "Unknown exception in the main loop. Aborting..." << std::endl;
         if (errno)
             perror("Possible cause");
@@ -363,11 +348,6 @@ int main ( int argc, char **argv )
 #if defined(HAVE_QT)
     flightgear::shutdownQtApp();
 #endif
-
-#if defined(HAVE_CRASHRPT)
-	crUninstall();
-#endif
-
     return exitStatus;
 }
 
@@ -380,6 +360,8 @@ void fgExitCleanup() {
         fgOSCloseWindow();
     }
 
+    flightgear::NavDataCache::shutdown();
+
     // you might imagine we'd call shutdownQtApp here, but it's not safe to do
     // so in an atexit handler, and crashes on Mac. Thiago states this explicitly:
     // https://bugreports.qt.io/browse/QTBUG-48709
@@ -393,4 +375,5 @@ void fgExitCleanup() {
     simgear::GroundLightManager::instance()->getGroundLightStateSet()->clear();
 
     simgear::shutdownLogging();
+    flightgear::shutdownSentry();
 }
