@@ -11,16 +11,18 @@
 #include <string>
 #include <fstream>
 
-#include <simgear/sg_inlines.h>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/math/sg_random.h>
 #include <simgear/io/iostreams/sgstream.hxx>
-#include <simgear/structure/exception.hxx>
-#include <simgear/structure/commands.hxx>
+#include <simgear/math/sg_random.h>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/sg_inlines.h>
+#include <simgear/structure/commands.hxx>
 #include <simgear/structure/event_mgr.hxx>
+#include <simgear/structure/exception.hxx>
 #include <simgear/timing/sg_time.hxx>
+
 #include <Network/RemoteXMLRequest.hxx>
 
 #include <FDM/flight.hxx>
@@ -33,6 +35,7 @@
 #include <Environment/presets.hxx>
 #include <Navaids/NavDataCache.hxx>
 #include <GUI/gui.h>
+#include <Main/sentryIntegration.hxx>
 
 #include "fg_init.hxx"
 #include "fg_io.hxx"
@@ -45,8 +48,6 @@
 #include "main.hxx"
 #include "positioninit.hxx"
 
-#include <boost/scoped_array.hpp>
-
 #if FG_HAVE_GPERFTOOLS
 # include <google/profiler.h>
 #endif
@@ -196,7 +197,13 @@ do_null (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_nasal (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    return ((FGNasalSys*)globals->get_subsystem("nasal"))->handleCommand(arg, root);
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
+    if (!nasalSys) {
+        SG_LOG(SG_GUI, SG_ALERT, "do_nasal command: Nasal subsystem not found");
+        return false;
+    }
+
+    return nasalSys->handleCommand(arg, root);
 }
 
 /**
@@ -430,7 +437,7 @@ do_property_adjust (const SGPropertyNode * arg, SGPropertyNode * root)
   if (arg->hasValue("step"))
       amount = arg->getDoubleValue("step");
   else
-      amount = (arg->getDoubleValue("factor", 1)
+      amount = (arg->getDoubleValue("factor", 1.0)
                 * arg->getDoubleValue("offset"));
 
   double unmodifiable, modifiable;
@@ -463,7 +470,7 @@ static bool
 do_property_multiply (const SGPropertyNode * arg, SGPropertyNode * root)
 {
   SGPropertyNode * prop = get_prop(arg,root);
-  double factor = arg->getDoubleValue("factor", 1);
+  double factor = arg->getDoubleValue("factor", 1.0);
 
   double unmodifiable, modifiable;
   split_value(prop->getDoubleValue(), arg->getStringValue("mask", "all"),
@@ -737,6 +744,10 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
     if (file.extension() != "xml")
         file.concat(".xml");
 
+    // some Nasal uses loadxml to also speculatively probe for existence of
+    // files. This flag allows us not to be noisy in the logs, in that case.
+    const bool quiet = arg->getBoolValue("quiet", false);
+
     std::string icao = arg->getStringValue("icao");
     if (icao.empty()) {
         if (file.isRelative()) {
@@ -745,23 +756,37 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
               file = absPath;
           else
           {
-              SG_LOG(SG_IO, SG_ALERT, "loadxml: Cannot find XML property file '"
-                          << file << "'.");
+              if (!quiet) {
+                  SG_LOG(SG_IO, SG_ALERT, "loadxml: Cannot find XML property file '" << file << "'.");
+                  simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLLoadCommand,
+                                         "loadxml: no such file:" + file.utf8Str(), file);
+              }
               return false;
           }
         }
     } else {
         if (!XMLLoader::findAirportData(icao, file.utf8Str(), file)) {
-          SG_LOG(SG_IO, SG_INFO, "loadxml: failed to find airport data for "
-            << file << " at ICAO:" << icao);
+            if (!quiet) {
+                SG_LOG(SG_IO, SG_INFO, "loadxml: failed to find airport data for " << file << " at ICAO:" << icao);
+                // disable this error, too noisy until jetways.xml is fixed
+                // simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLLoadCommand,
+                //                        "loadxml: no airport data file for:" + icao, file);
+            }
           return false;
         }
     }
 
+    if (!file.exists()) {
+        if (!quiet) {
+            SG_LOG(SG_IO, SG_WARN, "loadxml: no such file:" << file);
+        }
+        return false;
+    }
+
     SGPath validated_path = fgValidatePath(file, false);
     if (validated_path.isNull()) {
-        SG_LOG(SG_IO, SG_ALERT, "loadxml: reading '" << file << "' denied "
-                "(unauthorized directory - authorization no longer follows symlinks; to authorize reading additional directories, pass them to --allow-nasal-read)");
+        SG_LOG(SG_IO, quiet ? SG_DEV_WARN : SG_ALERT, "loadxml: reading '" << file << "' denied "
+                                                                                      "(unauthorized directory - authorization no longer follows symlinks; to authorize reading additional directories, pass them to --allow-nasal-read)");
         return false;
     }
 
@@ -774,7 +799,11 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
     try {
         readProperties(validated_path, targetnode, true);
     } catch (const sg_exception &e) {
-        SG_LOG(SG_IO, SG_WARN, "loadxml: " << e.getFormattedMessage());
+        if (!quiet) {
+            simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::XMLLoadCommand,
+                                   "loadxml exception:" + e.getFormattedMessage(), e.getLocation());
+        }
+        SG_LOG(SG_IO, quiet ? SG_DEV_WARN : SG_WARN, "loadxml exception: " << e.getFormattedMessage());
         return false;
     }
 
@@ -972,6 +1001,10 @@ static struct {
 void
 fgInitCommands ()
 {
+  // set our property root as the implicit default root for the
+  // command managr
+  SGCommandMgr::instance()->setImplicitRoot(globals->get_props());
+    
   SG_LOG(SG_GENERAL, SG_BULK, "Initializing basic built-in commands:");
   for (int i = 0; built_ins[i].name != 0; i++) {
     SG_LOG(SG_GENERAL, SG_BULK, "  " << built_ins[i].name);
