@@ -22,13 +22,10 @@
 
 #include <config.h>
 
-#include <boost/foreach.hpp>
 #include <algorithm>
 
-#ifndef FG_TESTLIB
 #include <osgViewer/Viewer>
 #include <osgDB/Registry>
-#endif
 
 #include <simgear/structure/commands.hxx>
 #include <simgear/structure/exception.hxx>
@@ -42,6 +39,7 @@
 #include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/propertyObject.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/props/AtomicChangeListener.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/package/Root.hxx>
 
@@ -52,9 +50,9 @@
 #include <Navaids/navlist.hxx>
 
 #include <GUI/gui.h>
+#include <Main/sentryIntegration.hxx>
 #include <Viewer/viewmgr.hxx>
 
-#ifndef FG_TESTLIB
 #include <Scenery/scenery.hxx>
 #include <Scenery/tilemgr.hxx>
 #include <Viewer/renderer.hxx>
@@ -63,7 +61,6 @@
 
 #include <simgear/sound/soundmgr.hxx>
 #include <simgear/scene/material/matlib.hxx>
-#endif
 
 #include "globals.hxx"
 #include "locale.hxx"
@@ -138,6 +135,17 @@ public:
   }
 };
 
+class CompositorEffectsProvider : public simgear::ResourceProvider {
+public:
+    CompositorEffectsProvider() :
+        simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {
+    }
+    SGPath resolve(const std::string &aResource, SGPath&) const override {
+        const SGPath p = globals->get_fg_root() / "Compositor" / aResource;
+        return p.exists() ? p : SGPath();
+    }
+};
+
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGGlobals.
 ////////////////////////////////////////////////////////////////////////
@@ -148,9 +156,7 @@ FGGlobals *globals = NULL;
 
 // Constructor
 FGGlobals::FGGlobals() :
-#ifndef FG_TESTLIB
     renderer( new FGRenderer ),
-#endif
     subsystem_mgr( new SGSubsystemMgr ),
     event_mgr( new SGEventMgr ),
     sim_time_sec( 0.0 ),
@@ -171,6 +177,9 @@ FGGlobals::FGGlobals() :
     resMgr->addProvider(new AircraftResourceProvider());
     resMgr->addProvider(new CurrentAircraftDirProvider());
     resMgr->addProvider(new flightgear::addons::ResourceProvider());
+#ifdef ENABLE_COMPOSITOR
+    resMgr->addProvider(new CompositorEffectsProvider());
+#endif
     initProperties();
 }
 
@@ -200,20 +209,22 @@ FGGlobals::~FGGlobals()
 
     // stop OSG threading first, to avoid thread races while we tear down
     // scene-graph pieces
-#ifndef FG_TESTLIB
-    osg::ref_ptr<osgViewer::Viewer> vw(renderer->getViewer());
-    if (vw) {
-        // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
-        // explicitly stop trheading before we delete the renderer or
-        // viewMgr (which ultimately holds refs to the CameraGroup, and
-        // GraphicsContext)
-        vw->stopThreading();
+    // there are some scenarios where renderer is already gone.
+    osg::ref_ptr<osgViewer::Viewer> vw;
+    if (renderer) {
+        vw = renderer->getViewer();
+        if (vw) {
+            // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
+            // explicitly stop trheading before we delete the renderer or
+            // viewMgr (which ultimately holds refs to the CameraGroup, and
+            // GraphicsContext)
+            vw->stopThreading();
+        }
     }
-#endif
+
     subsystem_mgr->shutdown();
     subsystem_mgr->unbind();
 
-#ifndef FG_TESTLIB
     // don't cancel the pager until after shutdown, since AIModels (and
     // potentially others) can queue delete requests on the pager.
     if (vw && vw->getDatabasePager()) {
@@ -222,31 +233,35 @@ FGGlobals::~FGGlobals()
     }
 
     osgDB::Registry::instance()->clearObjectCache();
-    subsystem_mgr->remove(FGScenery::subsystemName());
+    if (subsystem_mgr->get_subsystem(FGScenery::staticSubsystemClassId())) {
+        subsystem_mgr->remove(FGScenery::staticSubsystemClassId());
+    }
 
     // renderer touches subsystems during its destruction
-    set_renderer(NULL);
+    set_renderer(nullptr);
 
     FGFontCache::shutdown();
     fgCancelSnapShot();
-#endif
 
     delete subsystem_mgr;
-    subsystem_mgr = NULL; // important so ::get_subsystem returns NULL
-#ifndef FG_TESTLIB
-    vw = nullptr; // don't delete the viewer until now
+    subsystem_mgr = nullptr; // important so ::get_subsystem returns NULL
+    vw = nullptr;
     set_matlib(NULL);
-#endif
 
     delete time_params;
     delete channel_options_list;
     delete initial_waypoints;
     delete channellist;
 
+    // delete commands before we release the property root
+    // this avoids crash where commands might be storing a propery
+    // ref/pointer.
+    // see https://sentry.io/organizations/flightgear/issues/1890563449
+    delete commands;
+    commands = nullptr;
+    
     simgear::PropertyObjectBase::setDefaultRoot(NULL);
-#ifndef FG_TESTLIB
     simgear::SGModelLib::resetPropertyRoot();
-#endif
     delete locale;
     locale = NULL;
 
@@ -254,7 +269,7 @@ FGGlobals::~FGGlobals()
 
     props.clear();
 
-    delete commands;
+    delete simgear::ResourceManager::instance();
 }
 
 // set the fg_root path
@@ -291,17 +306,28 @@ void FGGlobals::set_fg_home (const SGPath &home)
     fg_home = home.realpath();
 }
 
+void FGGlobals::set_texture_cache_dir(const SGPath &textureCache)
+{
+    texture_cache_dir = textureCache.realpath();
+    auto node = fgGetNode("/sim/rendering/texture-cache/dir", true);
+    node->setAttribute(SGPropertyNode::WRITE, true);
+    node->setStringValue(textureCache.utf8Str());
+    node->setAttribute(SGPropertyNode::WRITE, false);
+}
+
+
 PathList FGGlobals::get_data_paths() const
 {
     PathList r(additional_data_paths);
     r.push_back(fg_root);
+    r.insert(r.end(), _dataPathsAfterFGRoot.begin(), _dataPathsAfterFGRoot.end());
     return r;
 }
 
 PathList FGGlobals::get_data_paths(const std::string& suffix) const
 {
     PathList r;
-    BOOST_FOREACH(SGPath p, get_data_paths()) {
+    for (SGPath p : get_data_paths()) {
         p.append(suffix);
         if (p.exists()) {
             r.push_back(p);
@@ -311,18 +337,28 @@ PathList FGGlobals::get_data_paths(const std::string& suffix) const
     return r;
 }
 
-void FGGlobals::append_data_path(const SGPath& path)
+void FGGlobals::append_data_path(const SGPath& path, bool afterFGRoot)
 {
     if (!path.exists()) {
         SG_LOG(SG_GENERAL, SG_WARN, "adding non-existant data path:" << path);
     }
 
-    additional_data_paths.push_back(path);
+    using RM = simgear::ResourceManager;
+    auto resManager = RM::instance();
+    if (afterFGRoot) {
+        _dataPathsAfterFGRoot.push_back(path);
+        // after FG_ROOT
+        resManager->addBasePath(path, static_cast<RM::Priority>(RM::PRIORITY_DEFAULT - 10));
+    } else {
+        additional_data_paths.push_back(path);
+        // after NORMAL prioirty, but ahead of FG_ROOT
+        resManager->addBasePath(path, static_cast<RM::Priority>(RM::PRIORITY_DEFAULT + 10));
+    }
 }
 
-SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
+SGPath FGGlobals::findDataPath(const std::string& pathSuffix) const
 {
-    BOOST_FOREACH(SGPath p, additional_data_paths) {
+    for (SGPath p : additional_data_paths) {
         p.append(pathSuffix);
         if (p.exists()) {
             return p;
@@ -335,8 +371,15 @@ SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
         return rootPath;
     }
 
-    SG_LOG(SG_GENERAL, SG_WARN, "dir not found in any data path:" << pathSuffix);
-    return SGPath();
+    for (SGPath p : _dataPathsAfterFGRoot) {
+        p.append(pathSuffix);
+        if (p.exists()) {
+            return p;
+        }
+    }
+
+    SG_LOG(SG_IO, SG_WARN, "not found in any data path: '" << pathSuffix << "'");
+    return SGPath{};
 }
 
 void FGGlobals::append_fg_scenery (const PathList &paths)
@@ -371,9 +414,7 @@ void FGGlobals::append_fg_scenery (const SGPath &path)
 
     // tell the ResouceManager about the scenery path
     // needed to load Models from this scenery path
-    simgear::ResourceManager::instance()->addBasePath(abspath.local8BitStr(),
-                                                      simgear::ResourceManager::PRIORITY_DEFAULT);
-    simgear::Dir dir(abspath);
+    simgear::ResourceManager::instance()->addBasePath(abspath, simgear::ResourceManager::PRIORITY_DEFAULT);
 
     fg_scenery.push_back(abspath);
     extra_read_allowed_paths.push_back(abspath);
@@ -391,7 +432,7 @@ void FGGlobals::append_read_allowed_paths(const SGPath &path)
 {
     SGPath abspath(path.realpath());
     if (!abspath.exists()) {
-        SG_LOG(SG_GENERAL, SG_WARN, "read-allowed path not found:" << abspath);
+        SG_LOG(SG_IO, SG_DEBUG, "read-allowed path not found:" << abspath);
         return;
     }
     extra_read_allowed_paths.push_back(abspath);
@@ -432,7 +473,7 @@ void FGGlobals::set_terrasync_dir(const SGPath &path)
 {
   if (terrasync_dir.realpath() != SGPath(fgGetString("/sim/terrasync/scenery-dir")).realpath()) {
     // if they don't match, /sim/terrasync/scenery-dir has been set by something else
-    SG_LOG(SG_GENERAL, SG_ALERT, "/sim/terrasync/scenery-dir is no longer stored across runs: if you wish to keep using a non-standard Terrasync directory, use --terrasync-dir or the launcher's settings");
+    SG_LOG(SG_GENERAL, SG_WARN, "/sim/terrasync/scenery-dir is no longer stored across runs: if you wish to keep using a non-standard Terrasync directory, use --terrasync-dir or the launcher's settings");
   }
   SGPath abspath(path.realpath());
   terrasync_dir = abspath;
@@ -467,7 +508,7 @@ void FGGlobals::append_aircraft_path(const SGPath& path)
 {
   SGPath dirPath(path);
   if (!dirPath.exists()) {
-    SG_LOG(SG_GENERAL, SG_ALERT, "aircraft path not found:" << path);
+    SG_LOG(SG_GENERAL, SG_WARN, "aircraft path not found:" << path);
     return;
   }
 
@@ -518,14 +559,12 @@ FGGlobals::get_renderer () const
 
 void FGGlobals::set_renderer(FGRenderer *render)
 {
-  #ifndef FG_TESTLIB
     if (render == renderer) {
         return;
     }
 
     delete renderer;
     renderer = render;
-#endif
 }
 
 SGSubsystemMgr *
@@ -623,6 +662,9 @@ FGGlobals::resetPropertyRoot()
 {
     delete locale;
 
+    // avoid a warning when we processOptions after reset
+    terrasync_dir = SGPath{};
+
     cleanupListeners();
 
     // we don't strictly need to clear these (they will be reset when we
@@ -773,17 +815,20 @@ static void tryAutosaveMigration(const SGPath& userDataPath, SGPropertyNode* pro
     // apply migration filters for each property in turn
     deleteProperties(&oldProps, blacklist);
 
+    // manual migrations
+    // migrate pre-2019.1 sense of /sim/mouse/invert-mouse-wheel
+    if (foundVersion.first < 2019) {
+        SGPropertyNode_ptr wheelNode = oldProps.getNode("/sim/mouse/invert-mouse-wheel");
+        if (wheelNode) {
+            wheelNode->setBoolValue(!wheelNode->getBoolValue());
+        }
+    }
     // copy remaining props out
     copyProperties(&oldProps, props);
 
-    // inform the user
-#if !defined(FG_TESTLIB)
-    flightgear::modalMessageBox("Settings migrated",
-                                "Saved settings were migrated from a previous version of FlightGear. "
-                                "If you encounter any problems when using the system, try restoring "
-                                "the default settings, before reporting a problem. "
-                                "Saved settings can affect the appearance, performance and features of the simulator.");
-#endif
+    // we can't inform the user yet, becuase embedded resources and the locale
+    // are not done. So we set a flag and check it once those things are done.
+    fgSetBool("/sim/autosave-migration/did-migrate", true);
 }
 
 // Load user settings from the autosave file (normally in $FG_HOME)
@@ -811,6 +856,13 @@ FGGlobals::loadUserSettings(SGPath userDataPath)
     } else {
         tryAutosaveMigration(userDataPath, &autosave);
     }
+    /* Before 2020-03-10, we could save portions of the /ai/models/ tree, which
+    confuses things when loaded again. So delete any such items if they have
+    been loaded. */
+    SGPropertyNode* ai = autosave.getNode("ai");
+    if (ai) {
+        ai->removeChildren("models");
+    }
     copyProperties(&autosave, globals->get_props());
 }
 
@@ -870,11 +922,7 @@ void FGGlobals::set_warp_delta( long int d )
 
 FGScenery* FGGlobals::get_scenery () const
 {
-#ifdef FG_TESTLIB
-    return nullptr;
-#else
     return get_subsystem<FGScenery>();
-#endif
 }
 
 FGViewMgr *FGGlobals::get_viewmgr() const
@@ -890,9 +938,7 @@ flightgear::View* FGGlobals::get_current_view () const
 
 void FGGlobals::set_matlib( SGMaterialLib *m )
 {
-#ifndef FG_TESTLIB
     matlib = m;
-#endif
 }
 
 FGControls *FGGlobals::get_controls() const
@@ -912,6 +958,8 @@ void FGGlobals::cleanupListeners()
         delete *i;
     }
     _listeners_to_cleanup.clear();
+
+    simgear::AtomicChangeListener::clearPendingChanges();
 }
 
 simgear::pkg::Root* FGGlobals::packageRoot()
@@ -924,4 +972,14 @@ void FGGlobals::setPackageRoot(const SGSharedPtr<simgear::pkg::Root>& p)
   _packageRoot = p;
 }
 
+bool FGGlobals::is_headless()
+{
+    return flightgear::isHeadlessMode();
+}
+
+void FGGlobals::set_headless(bool mode)
+{
+    flightgear::setHeadlessMode(mode);
+}
+
 // end of globals.cxx
