@@ -20,10 +20,7 @@
 //
 // $Id$
 
-
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <simgear/compiler.h>
 #include <simgear/props/props_io.hxx>
@@ -34,13 +31,6 @@
 #include <osg/GraphicsContext>
 #include <osgDB/Registry>
 
-#if defined(HAVE_CRASHRPT)
-	#include <CrashRpt.h>
-
-// defined in bootstrap.cxx
-extern bool global_crashRptEnabled;
-
-#endif
 
 // Class references
 #include <simgear/canvas/VGInitOperation.hxx>
@@ -55,6 +45,9 @@ extern bool global_crashRptEnabled;
 #include <simgear/math/SGMath.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/strutils.hxx>
+#include <simgear/structure/commands.hxx>
+#include <simgear/emesary/Emesary.hxx>
+#include <simgear/emesary/notifications.hxx>
 
 #include <Add-ons/AddonManager.hxx>
 #include <Main/locale.hxx>
@@ -69,18 +62,21 @@ extern bool global_crashRptEnabled;
 #include <Viewer/WindowSystemAdapter.hxx>
 #include <Navaids/NavDataCache.hxx>
 #include <Include/version.h>
+#include <Include/build.h>
 
 #include "fg_commands.hxx"
-#include "fg_io.hxx"
-#include "main.hxx"
-#include "util.hxx"
 #include "fg_init.hxx"
+#include "fg_io.hxx"
 #include "fg_os.hxx"
 #include "fg_props.hxx"
+#include "main.hxx"
+#include "options.hxx"
 #include "positioninit.hxx"
 #include "screensaver_control.hxx"
 #include "subsystemFactory.hxx"
-#include "options.hxx"
+#include "util.hxx"
+#include <Main/ErrorReporter.hxx>
+#include <Main/sentryIntegration.hxx>
 
 #include <simgear/embedded_resources/EmbeddedResourceManager.hxx>
 #include <EmbeddedResources/FlightGear-resources.hxx>
@@ -89,6 +85,10 @@ extern bool global_crashRptEnabled;
 #include <GUI/QtLauncher.hxx>
 #endif
 
+#ifdef __OpenBSD__
+#include <sys/resource.h>
+#endif
+
 using namespace flightgear;
 
 using std::cerr;
@@ -99,11 +99,52 @@ using std::vector;
 extern int _bootstrap_OSInit;
 
 static SGPropertyNode_ptr frame_signal;
+static SGPropertyNode_ptr nasal_gc_threaded;
+static SGPropertyNode_ptr nasal_gc_threaded_wait;
 
-// What should we do when we have nothing else to do?  Let's get ready
-// for the next move and update the display?
+#ifdef NASAL_BACKGROUND_GC_THREAD
+extern "C" {
+    extern void startNasalBackgroundGarbageCollection();
+    extern void stopNasalBackgroundGarbageCollection();
+    extern void performNasalBackgroundGarbageCollection();
+    extern void awaitNasalGarbageCollectionComplete(bool can_wait);
+}
+#endif
+static  simgear::Notifications::MainLoopNotification mln_begin(simgear::Notifications::MainLoopNotification::Type::Begin);
+static  simgear::Notifications::MainLoopNotification mln_end(simgear::Notifications::MainLoopNotification::Type::End);
+static  simgear::Notifications::MainLoopNotification mln_started(simgear::Notifications::MainLoopNotification::Type::Started);
+static  simgear::Notifications::MainLoopNotification mln_stopped(simgear::Notifications::MainLoopNotification::Type::Stopped);
+static  simgear::Notifications::NasalGarbageCollectionConfigurationNotification *ngccn = nullptr;
+// This method is usually called after OSG has finished rendering a frame in what OSG calls an idle handler and
+// is reposonsible for invoking all of the relevant per frame processing; most of which is handled by subsystems.
 static void fgMainLoop( void )
 {
+    //
+    // the Nasal GC will automatically run when (during allocation) it discovers that more space is needed.
+    // This has a cost of between 5ms and 50ms (depending on the amount of currently active Nasal).
+    // The result is unscheduled and unpredictable pauses during normal operation when the garbage collector
+    // runs; which typically occurs at intervals between 1sec and 20sec.
+    // 
+    // The solution to this, which overall increases CPU load, is to use a thread to do this; as Nasal is thread safe
+    // so what we do is to launch the garbage collection at the end of the main loop and then wait for completion at the start of the
+    // next main loop.
+    // So although the overall CPU is increased it has little effect on the frame rate; if anything it is an overall benefit
+    // as there are no unscheduled long duration frames.
+    //
+    // The implementation appears to work fine without waiting for completion at the start of the frame - so
+    // this wait at the start can be disabled by setting the property /sim/nasal-gc-threaded-wait to false.
+
+    // first we see if the config has changed. The notification will return true from SetActive/SetWait when the
+    // value has been changed - and thus we notify the Nasal system that it should configure itself accordingly.
+    auto use_threaded_gc = nasal_gc_threaded->getBoolValue();
+    auto threaded_wait = nasal_gc_threaded_wait->getBoolValue();
+    bool notify_gc_config = false;
+    notify_gc_config = ngccn->SetActive(use_threaded_gc);
+    notify_gc_config |= ngccn->SetWait(threaded_wait);
+    if (notify_gc_config)
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+
+     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_begin);
 
     if (sglog().has_popup()) {
         std::string s = sglog().get_popup();
@@ -121,7 +162,11 @@ static void fgMainLoop( void )
     // update all subsystems
     globals->get_subsystem_mgr()->update(sim_dt);
 
+    // flush commands waiting in the queue
+    SGCommandMgr::instance()->executedQueuedCommands();
     simgear::AtomicChangeListener::fireChangeListeners();
+
+     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_end);
 }
 
 static void initTerrasync()
@@ -129,28 +174,12 @@ static void initTerrasync()
     // add the terrasync root as a data path so data can be retrieved from it
     // (even if we are in read-only mode)
     SGPath terraSyncDir(globals->get_terrasync_dir());
-    globals->append_data_path(terraSyncDir);
+    globals->append_data_path(terraSyncDir, false /* = ahead of FG_ROOT */);
 
     if (fgGetBool("/sim/fghome-readonly", false)) {
         return;
     }
 
-    // start TerraSync up now, so it can be synchronizing shared models
-    // and airports data in parallel with a nav-cache rebuild.
-    SGPath tsyncCache(terraSyncDir);
-    tsyncCache.append("terrasync-cache.xml");
-
-    // wipe the cache file if requested
-    if (flightgear::Options::sharedInstance()->isOptionSet("restore-defaults")) {
-        SG_LOG(SG_GENERAL, SG_INFO, "restore-defaults requested, wiping terrasync update cache at " <<
-               tsyncCache);
-        if (tsyncCache.exists()) {
-            tsyncCache.remove();
-        }
-    }
-
-    fgSetString("/sim/terrasync/cache-path", tsyncCache.utf8Str());
-
     // make fg-root dir available so existing Scenery data can be copied, and
     // hence not downloaded again.
     fgSetString("/sim/terrasync/installation-dir", (globals->get_fg_root() / "Scenery").utf8Str());
@@ -161,10 +190,18 @@ static void initTerrasync()
 
     terra_sync->bind();
     terra_sync->init();
+    
+    if (fgGetBool("/sim/terrasync/enabled")) {
+        flightgear::addSentryTag("terrasync", "enabled");
+    }
 }
 
 static void fgSetVideoOptions()
 {
+    SGPath userDataPath = globals->get_fg_home();
+    SGPath autosaveFile = globals->autosaveFilePath(userDataPath);
+    if (autosaveFile.exists()) return;
+
     std::string vendor = fgGetString("/sim/rendering/gl-vendor");
     SGPath path(globals->get_fg_root());
     path.append("Video");
@@ -192,17 +229,16 @@ static void fgSetVideoOptions()
                 SG_LOG(SG_INPUT, SG_WARN, "failed to read video settings:" << e.getMessage()
                 << "(from " << e.getOrigin() << ")");
             }
-
-            flightgear::Options* options = flightgear::Options::sharedInstance();
-            if (!options->isOptionSet("ignore-autosave")) {
-                globals->loadUserSettings();
-            }
         }
     }
 }
 
 static void checkOpenGLVersion()
 {
+    flightgear::addSentryTag("gl-version", fgGetString("/sim/rendering/gl-version"));
+    flightgear::addSentryTag("gl-renderer", fgGetString("/sim/rendering/gl-vendor"));
+    flightgear::addSentryTag("gl-vendor", fgGetString("/sim/rendering/gl-renderer"));
+    
 #if defined(SG_MAC)
     // Mac users can't upgrade their drivers, so complaining about
     // versions doesn't help them much
@@ -232,13 +268,26 @@ static void checkOpenGLVersion()
     } // of three parts
 }
 
-static void registerMainLoop()
+namespace flightgear {
+
+void registerMainLoop()
 {
     // stash current frame signal property
     frame_signal = fgGetNode("/sim/signals/frame", true);
+    nasal_gc_threaded = fgGetNode("/sim/nasal-gc-threaded", true);
+    nasal_gc_threaded_wait = fgGetNode("/sim/nasal-gc-threaded-wait", true);
     fgRegisterIdleHandler( fgMainLoop );
 }
 
+void unregisterMainLoopProperties()
+{
+    frame_signal.reset();
+    nasal_gc_threaded.reset();
+    nasal_gc_threaded_wait.reset();
+}
+
+} // namespace flightgear
+
 // This is the top level master main function that is registered as
 // our idle function
 
@@ -285,11 +334,14 @@ static void fgIdleFunction ( void ) {
             throw sg_exception("General initialization failed");
         }
 
+        // now we have commands up
+        flightgear::delayedSentryInit();
+
         ////////////////////////////////////////////////////////////////////
         // Initialize the property-based built-in commands
         ////////////////////////////////////////////////////////////////////
         fgInitCommands();
-				fgInitSceneCommands();
+        fgInitSceneCommands();
 
         flightgear::registerSubsystemCommands(globals->get_commands());
 
@@ -304,7 +356,7 @@ static void fgIdleFunction ( void ) {
         flightgear::initPosition();
         flightgear::initTowerLocationListener();
 
-        simgear::SGModelLib::init(globals->get_fg_root().local8BitStr(), globals->get_props());
+        simgear::SGModelLib::init(globals->get_fg_root().utf8Str(), globals->get_props());
 
         auto timeManager = globals->get_subsystem<TimeManager>();
         timeManager->init();
@@ -323,7 +375,16 @@ static void fgIdleFunction ( void ) {
         idle_state = 8; // from the next state on, reset & startup are identical
         SGTimeStamp st;
         st.stamp();
-        fgCreateSubsystems(isReset);
+
+        try {
+            fgCreateSubsystems(isReset);
+        } catch (std::exception& e) {
+            // attempt to trace location of illegal argument / invalid string
+            // position errors on startup
+            flightgear::sentryReportException(string{"Creating subsystems: caught:"} + e.what());
+            throw;
+        }
+
         SG_LOG(SG_GENERAL, SG_INFO, "Creating subsystems took:" << st.elapsedMSec());
         fgSplashProgress("binding-subsystems");
 
@@ -371,10 +432,17 @@ static void fgIdleFunction ( void ) {
         // We've finished all our initialization steps, from now on we
         // run the main loop.
         fgSetBool("sim/sceneryloaded", false);
-        registerMainLoop();
+        flightgear::registerMainLoop();
+
+        ngccn = new simgear::Notifications::NasalGarbageCollectionConfigurationNotification(nasal_gc_threaded->getBoolValue(), nasal_gc_threaded_wait->getBoolValue());
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_started);
+        
+        flightgear::addSentryBreadcrumb("entering main loop", "info");
     }
 
     if ( idle_state == 2000 ) {
+        flightgear::addSentryBreadcrumb("starting reset", "info");
         fgStartNewReset();
         idle_state = 2005;
     }
@@ -390,7 +458,7 @@ void fgInitSecureMode()
 {
     bool secureMode = true;
     if (Options::sharedInstance()->isOptionSet("allow-nasal-from-sockets")) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "\n!! Network connections allowed to use Nasal !!\n"
+        SG_LOG(SG_GENERAL, SG_MANDATORY_INFO, "\n!! Network connections allowed to use Nasal !!\n"
        "Network connections will be allowed full access to the simulator \n"
        "including running arbitrary scripts. Ensure you have adequate security\n"
         "(such as a firewall which blocks external connections).\n");
@@ -446,55 +514,130 @@ extern "C" {
 }
 #endif
 
-static void logToHome()
+static void rotateOldLogFiles()
 {
+    const int maxLogCount = 10;
+    const auto homePath = globals->get_fg_home();
+
+    for (int i = maxLogCount; i > 0; --i) {
+        const auto name = "fgfs_" + std::to_string(i - 1) + ".log";
+        SGPath curLogFile = homePath / name;
+        if (curLogFile.exists()) {
+            auto newName = "fgfs_" + std::to_string(i) + ".log";
+            curLogFile.rename(homePath / newName);
+        }
+    }
+
+    SGPath p = homePath / "fgfs.log";
+    if (!p.exists())
+        return;
+    SGPath log0Path = homePath / "fgfs_0.log";
+    p.rename(log0Path);
+}
+
+static void logToHome(const std::string& pri)
+{
+    sgDebugPriority fileLogLevel = SG_INFO;
+    // https://sourceforge.net/p/flightgear/codetickets/2100/
+    if (!pri.empty()) {
+        try {
+            fileLogLevel = std::min(fileLogLevel, logstream::priorityFromString(pri));
+        } catch (std::exception& e) {
+            // let's not worry about this, and just log at INFO
+        }
+    }
+
     SGPath logPath = globals->get_fg_home();
     logPath.append("fgfs.log");
     if (logPath.exists()) {
-        SGPath prevLogPath = globals->get_fg_home();
-        prevLogPath.append("fgfs_0.log");
-        logPath.rename(prevLogPath);
-        // bit strange, we need to restore the correct value of logPath now
-        logPath = globals->get_fg_home();
-        logPath.append("fgfs.log");
+        rotateOldLogFiles();
     }
-    sglog().logToFile(logPath, SG_ALL, SG_INFO);
 
-#if defined(HAVE_CRASHRPT)
-	if (global_crashRptEnabled) {
-		crAddFile2(logPath.c_str(), NULL, "FlightGear Log File", CR_AF_MAKE_FILE_COPY);
-		SG_LOG( SG_GENERAL, SG_INFO, "CrashRpt enabled");
-	} else {
-		SG_LOG(SG_GENERAL, SG_WARN, "CrashRpt enabled at compile time but failed to install");
-	}
-#endif
+    sglog().logToFile(logPath, SG_ALL, fileLogLevel);
 }
 
 // Main top level initialization
 int fgMainInit( int argc, char **argv )
 {
-    // set default log level to 'info' for startup, we will revert to a lower
-    // level once startup is done.
-    sglog().setLogLevels( SG_ALL, SG_INFO );
+    sglog().setLogLevels( SG_ALL, SG_WARN );
     sglog().setStartupLoggingEnabled(true);
-
+    
     globals = new FGGlobals;
-    if (!fgInitHome()) {
-        return EXIT_FAILURE;
+    auto initHomeResult = fgInitHome();
+    if (initHomeResult == InitHomeAbort) {
+        flightgear::fatalMessageBoxThenExit("Unable to create lock file",
+                                "Flightgear was unable to create the lock file in FG_HOME");
     }
+    
+#if defined(HAVE_QT)
+    flightgear::initApp(argc, argv);
+#endif
 
+    // check if the launcher is requested, since it affects config file parsing
+    bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
+    // an Info.plist bundle can't define command line arguments, but it can set
+    // environment variables. This avoids needed a wrapper shell-script on OS-X.
+    showLauncher |= (::getenv("FG_LAUNCHER") != nullptr);
+
+#if defined(HAVE_QT)
+    if (showLauncher && (initHomeResult == InitHomeReadOnly)) {
+// show this message early, if we can
+        auto r = flightgear::showLockFileDialog();
+        if (r == flightgear::LockFileReset) {
+            SG_LOG( SG_GENERAL, SG_MANDATORY_INFO, "Deleting lock file at user request");
+            flightgear::addSentryBreadcrumb("deleting lock-file at user request", "info");
+            fgDeleteLockFile();
+            fgSetBool("/sim/fghome-readonly", false);
+        } else if (r == flightgear::LockFileQuit) {
+            return EXIT_SUCCESS;
+        }
+    }
+#endif
+    
     const bool readOnlyFGHome = fgGetBool("/sim/fghome-readonly");
     if (!readOnlyFGHome) {
         // now home is initialised, we can log to a file inside it
-        logToHome();
+        const auto level = flightgear::Options::getArgValue(argc, argv, "--log-level");
+        logToHome(level);
+    }
+
+    if (readOnlyFGHome) {
+        flightgear::addSentryTag("fghome-readonly", "true");
     }
 
     std::string version(FLIGHTGEAR_VERSION);
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version " << version );
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Build Type " << FG_BUILD_TYPE );
     SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR);
-	SG_LOG( SG_GENERAL, SG_INFO, "Jenkins number/ID " << HUDSON_BUILD_NUMBER << ":"
-			<< HUDSON_BUILD_ID);
+	SG_LOG( SG_GENERAL, SG_INFO, "Jenkins number/ID " << JENKINS_BUILD_NUMBER << ":"
+			<< JENKINS_BUILD_ID);
+
+#if OSG_VERSION_LESS_THAN(3,4,1)
+    SG_LOG(SG_GENERAL, SG_ALERT, "Minimum supported OpenScenegraph is V3.4.1 - currently using " << osgGetVersion() << " This can cause fatal OSG 'final reference count' errors at runtime");
+#endif
+    
+#ifdef __OpenBSD__
+    {
+        /* OpenBSD defaults to a small maximum data segment, which can cause
+        flightgear to crash with SIGBUS, so output a warning if this is likely.
+        */
+        struct rlimit   rlimit;
+        int e = getrlimit(RLIMIT_DATA, &rlimit);
+        if (e) {
+            SG_LOG( SG_GENERAL, SG_INFO, "This is OpenBSD; getrlimit() failed: " << strerror(errno));
+        }
+        else {
+            long long   required = 4LL * (1LL<<30);
+            if (rlimit.rlim_cur < required) {
+                SG_LOG( SG_GENERAL, SG_POPUP, ""
+                        << "Max data segment (" << rlimit.rlim_cur << "bytes) too small.\n"
+                        << "This can cause Flightgear to crash due to SIGBUS.\n"
+                        << "E.g. increase with 'ulimit -d " << required/1024 << "'."
+                        );
+            }
+        }
+    }
+#endif
 
     // seed the random number generator
     sg_srandom_time();
@@ -508,11 +651,6 @@ int fgMainInit( int argc, char **argv )
     upper_case_property("/sim/tower/airport-id");
     upper_case_property("/autopilot/route-manager/input");
 
-// check if the launcher is requested, since it affects config file parsing
-    bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
-    // an Info.plist bundle can't define command line arguments, but it can set
-    // environment variables. This avoids needed a wrapper shell-script on OS-X.
-    showLauncher |= (::getenv("FG_LAUNCHER") != 0);
     if (showLauncher) {
         // to minimise strange interactions when launcher and config files
         // set overlaping options, we disable the default files. Users can
@@ -520,13 +658,6 @@ int fgMainInit( int argc, char **argv )
         flightgear::Options::sharedInstance()->setShouldLoadDefaultConfig(false);
     }
 
-    if (showLauncher && readOnlyFGHome) {
-        // this is perhaps not what the user wanted, let's inform them
-        flightgear::modalMessageBox("Multiple copies of FlightGear",
-                                    "Another copy of FlightGear is already running on this computer, "
-                                    "so this copy will run in read-only mode.");
-    }
-
     // Load the configuration parameters.  (Command line options
     // override config file options.  Config file options override
     // defaults.)
@@ -539,12 +670,12 @@ int fgMainInit( int argc, char **argv )
 
 #if defined(HAVE_QT)
     if (showLauncher) {
-        flightgear::initApp(argc, argv);
-        flightgear::checkKeyboardModifiersForSettingFGRoot();
-
+        flightgear::addSentryBreadcrumb("starting launcher", "info");
         if (!flightgear::runLauncherDialog()) {
             return EXIT_SUCCESS;
         }
+
+        flightgear::addSentryBreadcrumb("completed launcher", "info");
     }
 #else
     if (showLauncher) {
@@ -555,10 +686,15 @@ int fgMainInit( int argc, char **argv )
     fgInitSecureMode();
     fgInitAircraftPaths(false);
 
+    auto errorManager = globals->add_new_subsystem<flightgear::ErrorReporter>();
+    errorManager->preinit();
+
     configResult = fgInitAircraft(false);
     if (configResult == flightgear::FG_OPTIONS_ERROR) {
         return EXIT_FAILURE;
-    } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
+    } else if ((configResult == flightgear::FG_OPTIONS_EXIT) ||
+               (configResult == flightgear::FG_OPTIONS_SHOW_AIRCRAFT))
+    {
         return EXIT_SUCCESS;
     }
 
@@ -585,6 +721,19 @@ int fgMainInit( int argc, char **argv )
     SG_LOG(SG_GENERAL, SG_INFO,
            "EmbeddedResourceManager: selected locale '" << locale << "'");
 
+    if (fgGetBool("/sim/autosave-migration/did-migrate", false)) {
+        // inform the user we did migration. This is the earliest point
+        // we can do it, since now the locale is set
+        auto locale = globals->get_locale();
+        const auto title = locale->getLocalizedString("settings-migration-title", "sys", "Settings migrated");
+        const auto msg = locale->getLocalizedString("settings-migration-text", "sys",
+                                                    "Saved settings were migrated from a previous version of FlightGear. "
+                                                    "If you encounter any problems when using the system, try restoring "
+                                                    "the default settings, before reporting a problem. "
+                                                    "Saved settings can affect the appearance, performance and features of the simulator.");
+        flightgear::modalMessageBox(title, msg);
+    }
+
     // Copy the property nodes for the menus added by registered add-ons
     addons::AddonManager::instance()->addAddonMenusToFGMenubar();
 
@@ -604,8 +753,13 @@ int fgMainInit( int argc, char **argv )
     fntInit();
     globals->get_renderer()->preinit();
 
+#if defined(ENABLE_COMPOSITOR)
+    flightgear::addSentryTag("compositor", "yes");
+#endif
+
     if (fgGetBool("/sim/ati-viewport-hack", true)) {
         SG_LOG(SG_GENERAL, SG_WARN, "Enabling ATI/AMD viewport hack");
+        flightgear::addSentryTag("ati-viewport-hack", "enabled");
         ATIScreenSizeHack();
     }
 
@@ -616,19 +770,35 @@ int fgMainInit( int argc, char **argv )
 
     // pass control off to the master event handler
     int result = fgOSMainLoop();
-    frame_signal.clear();
+    flightgear::unregisterMainLoopProperties();
+
     fgOSCloseWindow();
+    fgShutdownHome();
+
+    const bool requestLauncherRestart = fgGetBool("/sim/restart-launcher-on-exit");
+
+    simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_stopped);
 
     simgear::clearEffectCache();
 
     // clean up here; ensure we null globals to avoid
     // confusing the atexit() handler
     delete globals;
-    globals = NULL;
+    globals = nullptr;
 
     // delete the NavCache here. This will cause the destruction of many cached
     // objects (eg, airports, navaids, runways).
     delete flightgear::NavDataCache::instance();
 
+#if defined(HAVE_QT)
+    if (requestLauncherRestart) {
+        string_list originalArgs;
+        for (int arg = 1; arg < argc; ++arg) {
+            originalArgs.push_back(argv[arg]);
+        }
+        flightgear::addSentryBreadcrumb("Requested to restart launcher", "info");
+        flightgear::startLaunchOnExit(originalArgs);
+    }
+#endif
     return result;
 }
