@@ -8,10 +8,11 @@
 #endif
 
 #include <cstring>		// for strcmp()
-#include <boost/foreach.hpp>
 
 #include <simgear/compiler.h>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/logstream.hxx>
+
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/scene/model/placement.hxx>
@@ -33,7 +34,7 @@ static osg::Node *
 fgLoad3DModelPanel(const SGPath &path, SGPropertyNode *prop_root)
 {
     bool loadPanels = true;
-    osg::Node* node = simgear::SGModelLib::loadModel(path.local8BitStr(), prop_root, NULL, loadPanels);
+    osg::Node* node = simgear::SGModelLib::loadModel(path.utf8Str(), prop_root, NULL, loadPanels);
     if (node)
         node->setNodeMask(~SG_NODEMASK_TERRAIN_BIT);
     return node;
@@ -61,33 +62,44 @@ FGAircraftModel::~FGAircraftModel ()
   shutdown();
 }
 
-void 
+void
 FGAircraftModel::init ()
 {
     if (_aircraft.get()) {
         SG_LOG(SG_AIRCRAFT, SG_ALERT, "FGAircraftModel::init: already inited");
         return;
     }
-    
+
+    simgear::ErrorReportContext ec("primary-aircraft", "yes");
+
     SGPropertyNode_ptr sim = fgGetNode("/sim", true);
-    BOOST_FOREACH(SGPropertyNode_ptr model, sim->getChildren("model")) {
+    for (auto model : sim->getChildren("model")) {
         std::string path = model->getStringValue("path", "Models/Geometry/glider.ac");
         std::string usage = model->getStringValue("usage", "external");
-        
+
+        simgear::ErrorReportContext ec("aircraft-model", path);
+
         SGPath resolvedPath = globals->resolve_aircraft_path(path);
         if (resolvedPath.isNull()) {
+            simgear::reportFailure(simgear::LoadFailure::NotFound,
+                                   simgear::ErrorCode::XMLModelLoad,
+                                   "Failed to find aircraft model", SGPath::fromUtf8(path));
             SG_LOG(SG_AIRCRAFT, SG_ALERT, "Failed to find aircraft model: " << path);
             continue;
         }
-        
+
         osg::Node* node = NULL;
         try {
             node = fgLoad3DModelPanel( resolvedPath, globals->get_props());
         } catch (const sg_exception &ex) {
+            simgear::reportFailure(simgear::LoadFailure::BadData,
+                                   simgear::ErrorCode::XMLModelLoad,
+                                   "Failed to load aircraft model:" + ex.getFormattedMessage(),
+                                   ex.getLocation());
             SG_LOG(SG_AIRCRAFT, SG_ALERT, "Failed to load aircraft from " << path << ':');
             SG_LOG(SG_AIRCRAFT, SG_ALERT, "  " << ex.getFormattedMessage());
         }
-        
+
         if (usage == "interior") {
             // interior model
             if (!_interior.get()) {
@@ -106,11 +118,11 @@ FGAircraftModel::init ()
             }
         } // of model usage switch
     } // of models iteration
-    
+
     // no models loaded, load the glider instead
     if (!_aircraft.get()) {
         SG_LOG(SG_AIRCRAFT, SG_ALERT, "(Falling back to glider.ac.)");
-        osg::Node* model = fgLoad3DModelPanel( SGPath::fromLocal8Bit("Models/Geometry/glider.ac"),
+        osg::Node* model = fgLoad3DModelPanel( SGPath::fromUtf8("Models/Geometry/glider.ac"),
                                    globals->get_props());
         _aircraft.reset(new SGModelPlacement);
         _aircraft->init(model);
@@ -119,7 +131,7 @@ FGAircraftModel::init ()
 
   osg::Node* node = _aircraft->getSceneGraph();
   globals->get_scenery()->get_aircraft_branch()->addChild(node);
-    
+
     if (_interior.get()) {
         node = _interior->getSceneGraph();
         globals->get_scenery()->get_interior_branch()->addChild(node);
@@ -174,21 +186,21 @@ FGAircraftModel::unbind ()
 void
 FGAircraftModel::update (double dt)
 {
-  int view_number = globals->get_viewmgr()->get_current();
-  int is_internal = fgGetBool("/sim/current-view/internal");
+    int view_number = globals->get_viewmgr()->getCurrentViewIndex();
+    int is_internal = fgGetBool("/sim/current-view/internal");
 
-  if (view_number == 0 && !is_internal) {
-    _aircraft->setVisible(false);
+    if (view_number == 0 && !is_internal) {
+        _aircraft->setVisible(false);
   } else {
     _aircraft->setVisible(true);
   }
-    
+
     double heading, pitch, roll;
     globals->get_aircraft_orientation(heading, pitch, roll);
     SGQuatd orient = SGQuatd::fromYawPitchRollDeg(heading, pitch, roll);
-    
+
     SGGeod pos = globals->get_aircraft_position();
-    
+
     _aircraft->setPosition(pos);
     _aircraft->setOrientation(orient);
     _aircraft->update();
@@ -198,16 +210,25 @@ FGAircraftModel::update (double dt)
         _interior->setOrientation(orient);
         _interior->update();
     }
-    
+
   // update model's audio sample values
   _fx->set_position_geod( pos );
   _fx->set_orientation( orient );
- 
+
   _velocity = SGVec3d( _speed_n->getDoubleValue(),
                        _speed_e->getDoubleValue(),
                        _speed_d->getDoubleValue() );
   _fx->set_velocity( _velocity );
+
+  float temp_c = fgGetFloat("/environment/temperature-degc");
+  float humidity = fgGetFloat("/environment/relative-humidity");
+  float pressure = fgGetFloat("/environment/pressure-inhg")*SG_INHG_TO_PA/1000.0f;
+  _fx->set_atmosphere( temp_c, humidity, pressure );
 }
 
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGAircraftModel> registrantFGAircraftModel(
+    SGSubsystemMgr::DISPLAY);
+
 // end of model.cxx
