@@ -2,12 +2,13 @@
 # include "config.h"
 #endif
 
-#include "LevelDXML.hxx"
+#include <algorithm>
 
-#include <boost/algorithm/string.hpp>
+#include "LevelDXML.hxx"
 
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/strutils.hxx>
 
 #include <Navaids/waypoint.hxx>
 #include <Airports/airport.hxx>
@@ -85,8 +86,13 @@ void NavdataVisitor::startElement(const char* name, const XMLAttributes &atts)
     _transWaypts.clear();
   } else if (tag == "RunwayTransition") {
     _transIdent = atts.getValue("Runway");
-    _transition = new Transition(_transIdent, PROCEDURE_RUNWAY_TRANSITION, _procedure);
-    _transWaypts.clear();
+      if (!_airport->hasRunwayWithIdent(_transIdent)) {
+          _transIdent.clear();
+          _transition = nullptr;
+      } else {
+          _transition = new Transition(_transIdent, PROCEDURE_RUNWAY_TRANSITION, _procedure);
+          _transWaypts.clear();
+      }
   } else {
     // nothing here, we warn on unrecognized in endElement
   }
@@ -106,11 +112,14 @@ void NavdataVisitor::processRunways(ArrivalDeparture* aProc, const XMLAttributes
     return;
   }
   
-  vector<string> rwys;
-  boost::split(rwys, v, boost::is_any_of(" ,"));
-  for (unsigned int r=0; r<rwys.size(); ++r) {
-    FGRunway* rwy = _airport->getRunwayByIdent(rwys[r]);
-    aProc->addRunway(rwy);
+  auto rwys = simgear::strutils::split_on_any_of(v, " ,");
+  for (auto rwy : rwys) {
+      if (!_airport->hasRunwayWithIdent(rwy)) {
+          SG_LOG(SG_NAVAID, SG_DEV_WARN, "Procedure file " << _path << " references unknown airport runway:" << rwy);
+          continue;
+      }
+          
+    aProc->addRunway(_airport->getRunwayByIdent(rwy));
   }
 }
 
@@ -143,6 +152,11 @@ void NavdataVisitor::endElement(const char* name)
     _transition->setPrimary(_transWaypts);
     _approach->addTransition(_transition);
   } else if (tag == "RunwayTransition") {
+      if (!_transition) {
+          // transition was skipped for some reason
+          return;
+      }
+      
     ArrivalDeparture* ad;
     if (_sid) {
       // SID waypoints are stored backwards, to share code with STARs
@@ -208,7 +222,9 @@ void NavdataVisitor::endElement(const char* name)
     _overflightWaypt = (_text == "Fly-over");
   } else if ((tag == "AltitudeCons") ||
              (tag == "BankLimit") ||
-             (tag == "Sp_Turn"))
+             (tag == "Sp_Turn") ||
+             (tag == "Airport") ||
+             (tag == "ProceduresDB"))
   {
     // ignored but don't warn
   } else {
@@ -225,8 +241,15 @@ Waypt* NavdataVisitor::buildWaypoint(RouteBase* owner)
     wp = new BasicWaypt(pos, _wayptName, owner);
   } else if (_wayptType == "Runway") {
     string ident = _wayptName.substr(2);
-    FGRunwayRef rwy = _airport->getRunwayByIdent(ident);
-    wp = new RunwayWaypt(rwy, owner);
+      if (_airport->hasRunwayWithIdent(ident)) {
+          FGRunwayRef rwy = _airport->getRunwayByIdent(ident);
+          wp = new RunwayWaypt(rwy, owner);
+      } else {
+          SG_LOG(SG_NAVAID, SG_DEV_WARN, "Missing runway " << ident << " reading " << _path);
+          SGGeod pos(SGGeod::fromDeg(_longitude, _latitude));
+          // fall back to a basic WP
+          wp = new BasicWaypt(pos, _wayptName, owner);
+      }
   } else if (_wayptType == "Hold") {
     SGGeod pos(SGGeod::fromDeg(_longitude, _latitude));
     Hold* h = new Hold(pos, _wayptName, owner);
@@ -301,7 +324,10 @@ void NavdataVisitor::finishApproach()
   }
   
   if (!rwy) {
-    throw sg_format_exception("Malformed approach, no runway waypt", _ident);
+      SG_LOG(SG_NAVAID, SG_DEV_WARN, "Parsing:" << _path << " found approach without a runway:" << _ident);
+      delete _approach;
+      _approach = nullptr;
+      return;
   }
   
   WayptVec primary(_waypoints.begin(), it);
